@@ -6,10 +6,11 @@ use std::collections::{BTreeMap, HashMap, HashSet};
 use std::fmt;
 use std::time::{Duration, Instant};
 
+use crate::history::ReadMarker;
 use crate::message::server_time;
 use crate::time::Posix;
 use crate::user::{Nick, NickRef};
-use crate::{config, ctcp, dcc, isupport, message, mode, Buffer, Server, User};
+use crate::{buffer, config, ctcp, dcc, isupport, message, mode, Server, User};
 use crate::{file_transfer, server};
 
 const HIGHLIGHT_BLACKOUT_INTERVAL: Duration = Duration::from_secs(5);
@@ -36,7 +37,11 @@ pub enum State {
 #[derive(Debug)]
 pub enum Notification {
     DirectMessage(User),
-    Highlight(User, String),
+    Highlight {
+        enabled: bool,
+        user: User,
+        channel: String,
+    },
     MonitoredOnline(Vec<User>),
     MonitoredOffline(Vec<Nick>),
 }
@@ -79,6 +84,8 @@ pub enum Event {
     Broadcast(Broadcast),
     Notification(message::Encoded, Nick, Notification),
     FileTransferRequest(file_transfer::ReceiveRequest),
+    UpdateReadMarker(String, ReadMarker),
+    JoinedChannel(String),
 }
 
 pub struct Client {
@@ -92,13 +99,14 @@ pub struct Client {
     users: HashMap<String, Vec<User>>,
     labels: HashMap<String, Context>,
     batches: HashMap<String, Batch>,
-    reroute_responses_to: Option<Buffer>,
+    reroute_responses_to: Option<buffer::Upstream>,
     registration_step: RegistrationStep,
     listed_caps: Vec<String>,
     supports_labels: bool,
     supports_away_notify: bool,
     supports_account_notify: bool,
     supports_extended_join: bool,
+    supports_read_marker: bool,
     highlight_blackout: HighlightBlackout,
     registration_required_channels: Vec<String>,
     isupport: HashMap<isupport::Kind, isupport::Parameter>,
@@ -151,6 +159,7 @@ impl Client {
             supports_away_notify: false,
             supports_account_notify: false,
             supports_extended_join: false,
+            supports_read_marker: false,
             highlight_blackout: HighlightBlackout::Blackout(Instant::now()),
             registration_required_channels: vec![],
             isupport: HashMap::new(),
@@ -179,7 +188,7 @@ impl Client {
         }
     }
 
-    fn send(&mut self, buffer: &Buffer, mut message: message::Encoded) {
+    fn send(&mut self, buffer: &buffer::Upstream, mut message: message::Encoded) {
         if self.supports_labels {
             use proto::Tag;
 
@@ -367,13 +376,17 @@ impl Client {
                     if contains("multi-prefix") {
                         requested.push("multi-prefix");
                     }
+                    if contains("draft/read-marker") {
+                        requested.push("draft/read-marker");
+                    }
 
                     if !requested.is_empty() {
                         // Request
                         self.registration_step = RegistrationStep::Req;
-                        let _ = self
-                            .handle
-                            .try_send(command!("CAP", "REQ", requested.join(" ")));
+
+                        for message in group_capability_requests(&requested) {
+                            let _ = self.handle.try_send(message);
+                        }
                     } else {
                         // If none requested, end negotiation
                         self.registration_step = RegistrationStep::End;
@@ -399,6 +412,9 @@ impl Client {
                 if caps.contains(&"extended-join") {
                     self.supports_extended_join = true;
                 }
+                if caps.contains(&"draft/read-marker") {
+                    self.supports_read_marker = true;
+                }
 
                 let supports_sasl = caps.iter().any(|cap| cap.contains("sasl"));
 
@@ -479,12 +495,14 @@ impl Client {
                 if newly_contains("multi-prefix") {
                     requested.push("multi-prefix");
                 }
+                if newly_contains("draft/read-marker") {
+                    requested.push("draft/read-marker");
+                }
 
                 if !requested.is_empty() {
-                    // Request
-                    let _ = self
-                        .handle
-                        .try_send(command!("CAP", "REQ", requested.join(" ")));
+                    for message in group_capability_requests(&requested) {
+                        let _ = self.handle.try_send(message);
+                    }
                 }
 
                 self.listed_caps.extend(new_caps);
@@ -506,6 +524,9 @@ impl Client {
                 if del_caps.contains(&"extended-join") {
                     self.supports_extended_join = false;
                 }
+                if del_caps.contains(&"draft/read-marker") {
+                    self.supports_read_marker = false;
+                }
 
                 self.listed_caps
                     .retain(|cap| !del_caps.iter().any(|del_cap| del_cap == cap));
@@ -633,13 +654,15 @@ impl Client {
                         }
 
                         // Highlight notification
-                        if message::reference_user_text(user.nickname(), self.nickname(), text)
-                            && self.highlight_blackout.allow_highlights()
-                        {
+                        if message::references_user_text(user.nickname(), self.nickname(), text) {
                             return Some(vec![Event::Notification(
                                 message.clone(),
                                 self.nickname().to_owned(),
-                                Notification::Highlight(user, channel.clone()),
+                                Notification::Highlight {
+                                    enabled: self.highlight_blackout.allow_highlights(),
+                                    user,
+                                    channel: channel.clone(),
+                                },
                             )]);
                         } else if user.nickname() == self.nickname() && context.is_some() {
                             // If we sent (echo) & context exists (we sent from this client), ignore
@@ -819,32 +842,36 @@ impl Client {
                 if user.nickname() == self.nickname() {
                     self.chanmap.insert(channel.clone(), Channel::default());
 
-                    if let Some(state) = self.chanmap.get_mut(channel) {
-                        // Sends WHO to get away state on users.
-                        if self.isupport.contains_key(&isupport::Kind::WHOX) {
-                            let fields = if self.supports_account_notify {
-                                "tcnfa"
+                    // Sends WHO to get away state on users if WHO poll is enabled.
+                    if self.config.who_poll_enabled {
+                        if let Some(state) = self.chanmap.get_mut(channel) {
+                            if self.isupport.contains_key(&isupport::Kind::WHOX) {
+                                let fields = if self.supports_account_notify {
+                                    "tcnfa"
+                                } else {
+                                    "tcnf"
+                                };
+
+                                let _ = self.handle.try_send(command!(
+                                    "WHO",
+                                    channel,
+                                    fields,
+                                    isupport::WHO_POLL_TOKEN.to_owned()
+                                ));
+
+                                state.last_who = Some(WhoStatus::Requested(
+                                    Instant::now(),
+                                    Some(isupport::WHO_POLL_TOKEN),
+                                ));
                             } else {
-                                "tcnf"
-                            };
-
-                            let _ = self.handle.try_send(command!(
-                                "WHO",
-                                channel,
-                                fields,
-                                isupport::WHO_POLL_TOKEN.to_owned()
-                            ));
-
-                            state.last_who = Some(WhoStatus::Requested(
-                                Instant::now(),
-                                Some(isupport::WHO_POLL_TOKEN),
-                            ));
-                        } else {
-                            let _ = self.handle.try_send(command!("WHO", channel));
-                            state.last_who = Some(WhoStatus::Requested(Instant::now(), None));
+                                let _ = self.handle.try_send(command!("WHO", channel));
+                                state.last_who = Some(WhoStatus::Requested(Instant::now(), None));
+                            }
+                            log::debug!("[{}] {channel} - WHO requested", self.server);
                         }
-                        log::debug!("[{}] {channel} - WHO requested", self.server);
                     }
+
+                    return Some(vec![Event::JoinedChannel(channel.clone())]);
                 } else if let Some(channel) = self.chanmap.get_mut(channel) {
                     let user = if self.supports_extended_join {
                         accountname.as_ref().map_or(user.clone(), |accountname| {
@@ -1225,12 +1252,30 @@ impl Client {
             Command::Numeric(RPL_ENDOFMONLIST, _) => {
                 return None;
             }
+            Command::MARKREAD(target, Some(timestamp)) => {
+                if let Some(read_marker) = timestamp
+                    .strip_prefix("timestamp=")
+                    .and_then(|timestamp| timestamp.parse::<ReadMarker>().ok())
+                {
+                    return Some(vec![Event::UpdateReadMarker(target.clone(), read_marker)]);
+                }
+            }
             _ => {}
         }
 
         Some(vec![Event::Single(message, self.nickname().to_owned())])
     }
 
+    pub fn send_markread(&mut self, target: &str, read_marker: ReadMarker) {
+        if self.supports_read_marker {
+            let _ = self.handle.try_send(command!(
+                "MARKREAD",
+                target.to_string(),
+                format!("timestamp={read_marker}"),
+            ));
+        }
+    }
+
     fn sync(&mut self) {
         self.channels = self.chanmap.keys().cloned().collect();
         self.users = self
@@ -1304,7 +1349,9 @@ impl Client {
             }
 
             let request = match state.last_who {
-                Some(WhoStatus::Done(last)) if !self.supports_away_notify => {
+                Some(WhoStatus::Done(last))
+                    if !self.supports_away_notify && self.config.who_poll_enabled =>
+                {
                     (now.duration_since(last) >= self.config.who_poll_interval)
                         .then_some(Request::Poll)
                 }
@@ -1424,12 +1471,18 @@ impl Map {
         }
     }
 
-    pub fn send(&mut self, buffer: &Buffer, message: message::Encoded) {
+    pub fn send(&mut self, buffer: &buffer::Upstream, message: message::Encoded) {
         if let Some(client) = self.client_mut(buffer.server()) {
             client.send(buffer, message);
         }
     }
 
+    pub fn send_markread(&mut self, server: &Server, target: &str, read_marker: ReadMarker) {
+        if let Some(client) = self.client_mut(server) {
+            client.send_markread(target, read_marker);
+        }
+    }
+
     pub fn join(&mut self, server: &Server, channels: &[String]) {
         if let Some(client) = self.client_mut(server) {
             client.join(channels);
@@ -1442,6 +1495,20 @@ impl Map {
         }
     }
 
+    pub fn exit(&mut self) -> HashSet<Server> {
+        self.0
+            .iter_mut()
+            .filter_map(|(server, state)| {
+                if let State::Ready(client) = state {
+                    client.quit(None);
+                    Some(server.clone())
+                } else {
+                    None
+                }
+            })
+            .collect()
+    }
+
     pub fn resolve_user_attributes<'a>(
         &'a self,
         server: &Server,
@@ -1521,12 +1588,12 @@ impl Map {
 
 #[derive(Debug, Clone)]
 pub enum Context {
-    Buffer(Buffer),
-    Whois(Buffer),
+    Buffer(buffer::Upstream),
+    Whois(buffer::Upstream),
 }
 
 impl Context {
-    fn new(message: &message::Encoded, buffer: Buffer) -> Self {
+    fn new(message: &message::Encoded, buffer: buffer::Upstream) -> Self {
         if let Command::WHOIS(_, _) = message.command {
             Self::Whois(buffer)
         } else {
@@ -1538,7 +1605,7 @@ impl Context {
         matches!(self, Self::Whois(_))
     }
 
-    fn buffer(self) -> Buffer {
+    fn buffer(self) -> buffer::Upstream {
         match self {
             Context::Buffer(buffer) => buffer,
             Context::Whois(buffer) => buffer,
@@ -1660,6 +1727,26 @@ pub enum WhoStatus {
     Done(Instant),
 }
 
+fn group_capability_requests<'a>(
+    capabilities: &'a [&'a str],
+) -> impl Iterator<Item = proto::Message> + 'a {
+    const MAX_LEN: usize = proto::format::BYTE_LIMIT - b"CAP REQ :\r\n".len();
+
+    capabilities
+        .iter()
+        .scan(0, |count, capability| {
+            // Capability + a space
+            *count += capability.len() + 1;
+
+            let chunk = *count / MAX_LEN;
+
+            Some((chunk, capability))
+        })
+        .into_group_map()
+        .into_values()
+        .map(|capabilities| command!("CAP", "REQ", capabilities.into_iter().join(" ")))
+}
+
 /// Group channels together into as few JOIN messages as possible
 fn group_joins<'a>(
     channels: &'a [String],
@@ -1732,5 +1819,5 @@ fn group_monitors(
     })
     .into_group_map()
     .into_values()
-    .map(|targets| command!("MONITOR", "+", targets.into_iter().join(","),))
+    .map(|targets| command!("MONITOR", "+", targets.into_iter().join(",")))
 }
