@@ -1,5 +1,5 @@
 use std::path::PathBuf;
-use std::string;
+use std::{str, string};
 
 use tokio_stream::wrappers::ReadDirStream;
 use tokio_stream::StreamExt;
@@ -18,27 +18,29 @@ pub use self::notification::Notifications;
 pub use self::proxy::Proxy;
 pub use self::server::Server;
 pub use self::sidebar::Sidebar;
+
+use crate::appearance::theme::Colors;
+use crate::appearance::{self, Appearance};
 use crate::audio::{self, Sound};
 use crate::environment::config_dir;
 use crate::server::Map as ServerMap;
-use crate::theme::Colors;
 use crate::{environment, Theme};
 
 pub mod buffer;
 pub mod channel;
 pub mod file_transfer;
-mod keys;
+pub mod keys;
 pub mod notification;
 pub mod proxy;
 pub mod server;
 pub mod sidebar;
 
 const CONFIG_TEMPLATE: &str = include_str!("../../config.toml");
-const DEFAULT_THEME_FILE_NAME: &str = "ferra.toml";
+const DEFAULT_THEME_NAME: &str = "ferra";
 
 #[derive(Debug, Clone, Default)]
 pub struct Config {
-    pub themes: Themes,
+    pub appearance: Appearance,
     pub servers: ServerMap,
     pub proxy: Option<Proxy>,
     pub font: Font,
@@ -78,21 +80,6 @@ pub struct Font {
     pub size: Option<u8>,
 }
 
-#[derive(Debug, Clone)]
-pub struct Themes {
-    pub default: Theme,
-    pub all: Vec<Theme>,
-}
-
-impl Default for Themes {
-    fn default() -> Self {
-        Self {
-            default: Theme::default(),
-            all: vec![Theme::default()],
-        }
-    }
-}
-
 impl Config {
     pub fn config_dir() -> PathBuf {
         let dir = environment::config_dir();
@@ -134,10 +121,32 @@ impl Config {
     pub async fn load() -> Result<Self, Error> {
         use tokio::fs;
 
+        #[derive(Deserialize, Debug)]
+        #[serde(untagged)]
+        pub enum ThemeKeys {
+            Static(String),
+            Dynamic { light: String, dark: String },
+        }
+
+        impl Default for ThemeKeys {
+            fn default() -> Self {
+                Self::Static(Default::default())
+            }
+        }
+
+        impl ThemeKeys {
+            pub fn keys(&self) -> (&str, Option<&str>) {
+                match self {
+                    ThemeKeys::Static(manual) => (manual, None),
+                    ThemeKeys::Dynamic { light, dark } => (light, Some(dark)),
+                }
+            }
+        }
+
         #[derive(Deserialize)]
         pub struct Configuration {
             #[serde(default)]
-            pub theme: String,
+            pub theme: ThemeKeys,
             pub servers: ServerMap,
             pub proxy: Option<Proxy>,
             #[serde(default)]
@@ -159,9 +168,14 @@ impl Config {
         }
 
         let path = Self::path();
+        if !path.try_exists()? {
+            return Err(Error::ConfigMissing {
+                has_yaml_config: has_yaml_config()?,
+            });
+        }
         let content = fs::read_to_string(path)
             .await
-            .map_err(|e| Error::Read(e.to_string()))?;
+            .map_err(|e| Error::LoadConfigFile(e.to_string()))?;
 
         let Configuration {
             theme,
@@ -181,10 +195,12 @@ impl Config {
 
         let loaded_notifications = notifications.load_sounds()?;
 
-        let themes = Self::load_themes(&theme).await.unwrap_or_default();
+        let appearance = Self::load_appearance(theme.keys())
+            .await
+            .unwrap_or_default();
 
         Ok(Config {
-            themes,
+            appearance,
             servers,
             font,
             proxy,
@@ -198,7 +214,7 @@ impl Config {
         })
     }
 
-    async fn load_themes(default_key: &str) -> Result<Themes, Error> {
+    async fn load_appearance(theme_keys: (&str, Option<&str>)) -> Result<Appearance, Error> {
         use tokio::fs;
 
         #[derive(Deserialize)]
@@ -224,7 +240,8 @@ impl Config {
         };
 
         let mut all = vec![];
-        let mut default = Theme::default();
+        let mut first_theme = Theme::default();
+        let mut second_theme = theme_keys.1.map(|_| Theme::default());
         let mut has_halloy_theme = false;
 
         let mut stream = ReadDirStream::new(fs::read_dir(Self::themes_dir()).await?);
@@ -237,14 +254,17 @@ impl Config {
                 continue;
             };
 
-            if file_name.ends_with(".toml") {
+            if let Some(file_name) = file_name.strip_suffix(".toml") {
                 if let Some(theme) = read_entry(entry).await {
-                    if file_name.strip_suffix(".toml").unwrap_or_default() == default_key
-                        || file_name == default_key
-                    {
-                        default = theme.clone();
+                    if file_name == theme_keys.0 {
+                        first_theme = theme.clone();
                     }
-                    if file_name == DEFAULT_THEME_FILE_NAME {
+
+                    if Some(file_name) == theme_keys.1 {
+                        second_theme = Some(theme.clone());
+                    }
+
+                    if file_name.to_lowercase() == DEFAULT_THEME_NAME {
                         has_halloy_theme = true;
                     }
 
@@ -257,7 +277,13 @@ impl Config {
             all.push(Theme::default());
         }
 
-        Ok(Themes { default, all })
+        let selected = if let Some(second_theme) = second_theme {
+            appearance::Selected::dynamic(first_theme, second_theme)
+        } else {
+            appearance::Selected::specific(first_theme)
+        };
+
+        Ok(Appearance { selected, all })
     }
 
     pub fn create_initial_config() {
@@ -294,8 +320,8 @@ pub fn random_nickname_with_seed<R: Rng>(rng: &mut R) -> String {
 }
 
 /// Has YAML configuration file.
-pub fn has_yaml_config() -> bool {
-    config_dir().join("config.yaml").exists()
+fn has_yaml_config() -> Result<bool, Error> {
+    Ok(config_dir().join("config.yaml").try_exists()?)
 }
 
 fn default_tooltip() -> bool {
@@ -305,15 +331,27 @@ fn default_tooltip() -> bool {
 #[derive(Debug, Error, Clone)]
 pub enum Error {
     #[error("config could not be read: {0}")]
-    Read(String),
+    LoadConfigFile(String),
+    #[error("command could not be run: {0}")]
+    ExecutePasswordCommand(String),
     #[error("{0}")]
     Io(String),
     #[error("{0}")]
     Parse(String),
     #[error("UTF8 parsing error: {0}")]
-    UI(#[from] string::FromUtf8Error),
+    StrUtf8Error(#[from] str::Utf8Error),
+    #[error("UTF8 parsing error: {0}")]
+    StringUtf8Error(#[from] string::FromUtf8Error),
     #[error(transparent)]
     LoadSounds(#[from] audio::LoadError),
+    #[error("Only one of password, password_file and password_command can be set.")]
+    DuplicatePassword,
+    #[error("Only one of nick_password, nick_password_file and nick_password_command can be set.")]
+    DuplicateNickPassword,
+    #[error("Exactly one of sasl.plain.password, sasl.plain.password_file or sasl.plain.password_command must be set.")]
+    DuplicateSaslPassword,
+    #[error("Config does not exist")]
+    ConfigMissing { has_yaml_config: bool },
 }
 
 impl From<std::io::Error> for Error {
