@@ -15,6 +15,8 @@ pub struct Server {
     pub nick_password: Option<String>,
     /// The client's NICKSERV password file.
     pub nick_password_file: Option<String>,
+    /// The client's NICKSERV password command.
+    pub nick_password_command: Option<String>,
     /// The server's NICKSERV IDENTIFY syntax.
     pub nick_identify_syntax: Option<IdentifySyntax>,
     /// Alternative nicknames for the client, if the default is taken.
@@ -76,6 +78,9 @@ pub struct Server {
     /// Commands which are executed once connected.
     #[serde(default)]
     pub on_connect: Vec<String>,
+    /// Enable WHO polling. Defaults to `true`.
+    #[serde(default = "default_who_poll_enabled")]
+    pub who_poll_enabled: bool,
     /// WHO poll interval for servers without away-notify.
     #[serde(
         default = "default_who_poll_interval",
@@ -143,6 +148,7 @@ impl Default for Server {
             nickname: Default::default(),
             nick_password: Default::default(),
             nick_password_file: Default::default(),
+            nick_password_command: Default::default(),
             nick_identify_syntax: Default::default(),
             alt_nicks: Default::default(),
             username: Default::default(),
@@ -165,6 +171,7 @@ impl Default for Server {
             root_cert_path: Default::default(),
             sasl: Default::default(),
             on_connect: Default::default(),
+            who_poll_enabled: default_who_poll_enabled(),
             who_poll_interval: default_who_poll_interval(),
             who_retry_interval: default_who_retry_interval(),
             monitor: Default::default(),
@@ -189,6 +196,8 @@ pub enum Sasl {
         password: Option<String>,
         /// Account password file
         password_file: Option<String>,
+        /// Account password command
+        password_command: Option<String>,
     },
     External {
         /// The path to PEM encoded X509 user certificate for external auth
@@ -277,6 +286,10 @@ fn default_ghost_sequence() -> Vec<String> {
     vec!["REGAIN".into()]
 }
 
+fn default_who_poll_enabled() -> bool {
+    true
+}
+
 fn default_who_poll_interval() -> Duration {
     Duration::from_secs(180)
 }
