@@ -3,31 +3,59 @@ use std::collections::{HashMap, HashSet};
 use chrono::{DateTime, Utc};
 use futures::future::BoxFuture;
 use futures::{future, Future, FutureExt};
-use itertools::Itertools;
 use tokio::time::Instant;
 
 use crate::history::{self, History};
 use crate::message::{self, Limit};
-use crate::time::Posix;
 use crate::user::Nick;
-use crate::{config, input};
-use crate::{server, Buffer, Config, Input, Server, User};
+use crate::{buffer, config, input};
+use crate::{server, Config, Input, Server, User};
 
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
 pub struct Resource {
-    pub server: server::Server,
     pub kind: history::Kind,
 }
 
+impl Resource {
+    pub fn logs() -> Self {
+        Self {
+            kind: history::Kind::Logs,
+        }
+    }
+
+    pub fn highlights() -> Self {
+        Self {
+            kind: history::Kind::Highlights,
+        }
+    }
+}
+
 #[derive(Debug)]
 pub enum Message {
-    Loaded(
-        server::Server,
+    LoadFull(history::Kind, Result<history::Loaded, history::Error>),
+    UpdatePartial(history::Kind, Result<history::Metadata, history::Error>),
+    UpdateReadMarker(
         history::Kind,
-        Result<Vec<crate::Message>, history::Error>,
+        history::ReadMarker,
+        Result<(), history::Error>,
     ),
-    Closed(server::Server, history::Kind, Result<(), history::Error>),
-    Flushed(server::Server, history::Kind, Result<(), history::Error>),
+    Closed(
+        history::Kind,
+        Result<Option<history::ReadMarker>, history::Error>,
+    ),
+    Flushed(history::Kind, Result<(), history::Error>),
+    Exited(
+        Vec<(
+            history::Kind,
+            Result<Option<history::ReadMarker>, history::Error>,
+        )>,
+    ),
+}
+
+pub enum Event {
+    Loaded(history::Kind),
+    Closed(history::Kind, Option<history::ReadMarker>),
+    Exited(Vec<(history::Kind, Option<history::ReadMarker>)>),
 }
 
 #[derive(Debug, Default)]
@@ -43,20 +71,18 @@ impl Manager {
 
         let added = added.into_iter().map(|resource| {
             async move {
-                history::load(&resource.server.clone(), &resource.kind.clone())
-                    .map(move |result| Message::Loaded(resource.server, resource.kind, result))
+                history::load(resource.kind.clone())
+                    .map(move |result| Message::LoadFull(resource.kind, result))
                     .await
             }
             .boxed()
         });
 
         let removed = removed.into_iter().filter_map(|resource| {
-            self.data
-                .untrack(&resource.server, &resource.kind)
-                .map(|task| {
-                    task.map(|result| Message::Closed(resource.server, resource.kind, result))
-                        .boxed()
-                })
+            self.data.untrack(&resource.kind).map(|task| {
+                task.map(|result| Message::Closed(resource.kind, result))
+                    .boxed()
+            })
         });
 
         let tasks = added.chain(removed).collect();
@@ -66,201 +92,194 @@ impl Manager {
         tasks
     }
 
-    pub fn update(&mut self, message: Message) {
+    pub fn update(&mut self, message: Message) -> Option<Event> {
         match message {
-            Message::Loaded(server, kind, Ok(messages)) => {
+            Message::LoadFull(kind, Ok(loaded)) => {
                 log::debug!(
-                    "loaded history for {kind} on {server}: {} messages",
-                    messages.len()
+                    "loaded history for {kind}: {} messages",
+                    loaded.messages.len()
                 );
-                self.data.loaded(server, kind, messages);
+                self.data.load_full(kind.clone(), loaded);
+                return Some(Event::Loaded(kind));
             }
-            Message::Loaded(server, kind, Err(error)) => {
-                log::warn!("failed to load history for {kind} on {server}: {error}");
+            Message::LoadFull(kind, Err(error)) => {
+                log::warn!("failed to load history for {kind}: {error}");
             }
-            Message::Closed(server, kind, Ok(_)) => {
-                log::debug!("closed history for {kind} on {server}",);
+            Message::Closed(kind, Ok(read_marker)) => {
+                log::debug!("closed history for {kind}",);
+                return Some(Event::Closed(kind, read_marker));
             }
-            Message::Closed(server, kind, Err(error)) => {
-                log::warn!("failed to close history for {kind} on {server}: {error}")
+            Message::Closed(kind, Err(error)) => {
+                log::warn!("failed to close history for {kind}: {error}")
             }
-            Message::Flushed(server, kind, Ok(_)) => {
-                log::debug!("flushed history for {kind} on {server}",);
+            Message::Flushed(kind, Ok(_)) => {
+                // Will cause flush loop if we emit a log every time we flush logs
+                if !matches!(kind, history::Kind::Logs) {
+                    log::debug!("flushed history for {kind}",);
+                }
             }
-            Message::Flushed(server, kind, Err(error)) => {
-                log::warn!("failed to flush history for {kind} on {server}: {error}")
+            Message::Flushed(kind, Err(error)) => {
+                log::warn!("failed to flush history for {kind}: {error}")
+            }
+            Message::UpdatePartial(kind, Ok(metadata)) => {
+                log::debug!("loaded metadata for {kind}");
+                self.data.update_partial(kind, metadata);
+            }
+            Message::UpdatePartial(kind, Err(error)) => {
+                log::warn!("failed to load metadata for {kind}: {error}");
+            }
+            Message::UpdateReadMarker(kind, read_marker, Ok(_)) => {
+                log::debug!("updated read marker for {kind} to {read_marker}");
+            }
+            Message::UpdateReadMarker(kind, read_marker, Err(error)) => {
+                log::warn!("failed to update read marker for {kind} to {read_marker}: {error}");
+            }
+            Message::Exited(results) => {
+                let mut output = vec![];
+
+                for (kind, result) in results {
+                    match result {
+                        Ok(marker) => {
+                            log::debug!("closed history for {kind}",);
+                            output.push((kind, marker));
+                        }
+                        Err(error) => {
+                            log::warn!("failed to close history for {kind}: {error}");
+                            output.push((kind, None));
+                        }
+                    }
+                }
+
+                return Some(Event::Exited(output));
             }
         }
+
+        None
     }
 
     pub fn tick(&mut self, now: Instant) -> Vec<BoxFuture<'static, Message>> {
         self.data.flush_all(now)
     }
 
-    pub fn close(
-        &mut self,
-        server: Server,
-        kind: history::Kind,
-    ) -> Option<impl Future<Output = ()>> {
-        let history = self.data.map.get_mut(&server)?.remove(&kind)?;
+    pub fn close(&mut self, kind: history::Kind) -> Option<impl Future<Output = Message>> {
+        let history = self.data.map.remove(&kind)?;
 
-        Some(async move {
-            match history.close().await {
-                Ok(_) => {
-                    log::debug!("closed history for {kind} on {server}",);
-                }
-                Err(error) => {
-                    log::warn!("failed to close history for {kind} on {server}: {error}");
-                }
-            }
-        })
+        Some(history.close().map(|result| Message::Closed(kind, result)))
     }
 
-    pub fn close_server(&mut self, server: Server) -> Option<impl Future<Output = ()>> {
-        let map = self.data.map.remove(&server)?;
-
-        Some(async move {
-            let tasks = map.into_iter().map(move |(kind, state)| {
-                let server = server.clone();
-                state.close().map(move |result| (server, kind, result))
-            });
-
-            let results = future::join_all(tasks).await;
-
-            for (server, kind, result) in results {
-                match result {
-                    Ok(_) => {
-                        log::debug!("closed history for {kind} on {server}",);
-                    }
-                    Err(error) => {
-                        log::warn!("failed to close history for {kind} on {server}: {error}");
-                    }
-                }
-            }
-        })
-    }
-
-    pub fn close_all(&mut self) -> impl Future<Output = ()> {
+    pub fn exit(&mut self) -> impl Future<Output = Message> {
         let map = std::mem::take(&mut self.data).map;
 
         async move {
-            let tasks = map.into_iter().flat_map(|(server, map)| {
-                map.into_iter().map(move |(kind, state)| {
-                    let server = server.clone();
-                    state.close().map(move |result| (server, kind, result))
-                })
-            });
+            let tasks = map
+                .into_iter()
+                .map(|(kind, state)| state.close().map(move |result| (kind, result)));
 
-            let results = future::join_all(tasks).await;
-
-            for (server, kind, result) in results {
-                match result {
-                    Ok(_) => {
-                        log::debug!("closed history for {kind} on {server}",);
-                    }
-                    Err(error) => {
-                        log::warn!("failed to close history for {kind} on {server}: {error}");
-                    }
-                }
-            }
+            Message::Exited(future::join_all(tasks).await)
         }
     }
 
-    pub fn record_input(&mut self, input: Input, user: User, channel_users: &[User]) {
+    pub fn record_input(
+        &mut self,
+        input: Input,
+        user: User,
+        channel_users: &[User],
+    ) -> Vec<impl Future<Output = Message>> {
+        let mut tasks = vec![];
+
         if let Some(messages) = input.messages(user, channel_users) {
             for message in messages {
-                self.record_message(input.server(), message);
+                tasks.extend(self.record_message(input.server(), message));
             }
         }
 
         if let Some(text) = input.raw() {
-            self.data.input.record(input.buffer(), text.to_string());
+            self.data.input.record(&input.buffer, text.to_string());
         }
+
+        tasks
     }
 
     pub fn record_draft(&mut self, draft: input::Draft) {
         self.data.input.store_draft(draft);
     }
 
-    pub fn record_message(&mut self, server: &Server, message: crate::Message) {
-        self.data.add_message(
-            server.clone(),
-            history::Kind::from(message.target.clone()),
-            message,
-        );
+    pub fn record_message(
+        &mut self,
+        server: &Server,
+        message: crate::Message,
+    ) -> Option<impl Future<Output = Message>> {
+        history::Kind::from_server_message(server.clone(), &message)
+            .and_then(|kind| self.data.add_message(kind, message))
     }
 
-    pub fn get_channel_messages(
-        &self,
-        server: &Server,
-        channel: &str,
-        limit: Option<Limit>,
-        buffer_config: &config::Buffer,
-    ) -> Option<history::View<'_>> {
-        self.data.history_view(
-            server,
-            &history::Kind::Channel(channel.to_string()),
-            limit,
-            buffer_config,
-        )
-    }
-
-    pub fn get_server_messages(
-        &self,
-        server: &Server,
-        limit: Option<Limit>,
-        buffer_config: &config::Buffer,
-    ) -> Option<history::View<'_>> {
+    pub fn record_log(
+        &mut self,
+        record: crate::log::Record,
+    ) -> Option<impl Future<Output = Message>> {
         self.data
-            .history_view(server, &history::Kind::Server, limit, buffer_config)
+            .add_message(history::Kind::Logs, crate::Message::log(record))
     }
 
-    pub fn get_query_messages(
+    pub fn record_highlight(
+        &mut self,
+        message: crate::Message,
+    ) -> Option<impl Future<Output = Message>> {
+        self.data.add_message(history::Kind::Highlights, message)
+    }
+
+    pub fn update_read_marker(
+        &mut self,
+        kind: impl Into<history::Kind>,
+        read_marker: history::ReadMarker,
+    ) -> Option<impl Future<Output = Message>> {
+        self.data.update_read_marker(kind, read_marker)
+    }
+
+    pub fn channel_joined(
+        &mut self,
+        server: Server,
+        channel: String,
+    ) -> Option<impl Future<Output = Message>> {
+        self.data.channel_joined(server, channel)
+    }
+
+    pub fn get_messages(
         &self,
-        server: &Server,
-        nick: &Nick,
+        kind: &history::Kind,
         limit: Option<Limit>,
         buffer_config: &config::Buffer,
     ) -> Option<history::View<'_>> {
-        self.data.history_view(
-            server,
-            &history::Kind::Query(nick.clone()),
-            limit,
-            buffer_config,
-        )
+        self.data.history_view(kind, limit, buffer_config)
     }
 
     pub fn get_unique_queries(&self, server: &Server) -> Vec<&Nick> {
-        let Some(map) = self.data.map.get(server) else {
-            return vec![];
-        };
-
-        let queries = map
+        let queries = self
+            .data
+            .map
             .keys()
             .filter_map(|kind| match kind {
-                history::Kind::Query(user) => Some(user),
+                history::Kind::Query(s, user) => (s == server).then_some(user),
                 _ => None,
             })
-            .unique()
             .collect::<Vec<_>>();
 
         queries
     }
 
-    pub fn has_unread(&self, server: &Server, kind: &history::Kind) -> bool {
+    pub fn has_unread(&self, kind: &history::Kind) -> bool {
         self.data
             .map
-            .get(server)
-            .and_then(|map| map.get(kind))
-            .map(|history| {
-                matches!(
-                    history,
-                    History::Partial {
-                        unread_message_count,
-                        ..
-                    } if *unread_message_count > 0
-                )
-            })
+            .get(kind)
+            .map(|history| history.has_unread())
+            .unwrap_or_default()
+    }
+
+    pub fn read_marker(&self, kind: &history::Kind) -> Option<history::ReadMarker> {
+        self.data
+            .map
+            .get(kind)
+            .map(|history| history.read_marker())
             .unwrap_or_default()
     }
 
@@ -270,24 +289,26 @@ impl Manager {
         broadcast: Broadcast,
         config: &Config,
         sent_time: DateTime<Utc>,
-    ) {
-        let map = self.data.map.entry(server.clone()).or_default();
-
-        let channels = map
+    ) -> Vec<impl Future<Output = Message>> {
+        let channels = self
+            .data
+            .map
             .keys()
             .filter_map(|kind| {
-                if let history::Kind::Channel(channel) = kind {
-                    Some(channel)
+                if let history::Kind::Channel(s, channel) = kind {
+                    (s == server).then_some(channel)
                 } else {
                     None
                 }
             })
             .cloned();
-        let mut queries = map
+        let mut queries = self
+            .data
+            .map
             .keys()
             .filter_map(|kind| {
-                if let history::Kind::Query(nick) = kind {
-                    Some(nick)
+                if let history::Kind::Query(s, nick) = kind {
+                    (s == server).then_some(nick)
                 } else {
                     None
                 }
@@ -388,12 +409,13 @@ impl Manager {
             }
         };
 
-        messages.into_iter().for_each(|message| {
-            self.record_message(server, message);
-        });
+        messages
+            .into_iter()
+            .filter_map(|message| self.record_message(server, message))
+            .collect()
     }
 
-    pub fn input<'a>(&'a self, buffer: &Buffer) -> input::Cache<'a> {
+    pub fn input<'a>(&'a self, buffer: &buffer::Upstream) -> input::Cache<'a> {
         self.data.input.get(buffer)
     }
 }
@@ -418,77 +440,75 @@ fn with_limit<'a>(
 
 #[derive(Debug, Default)]
 struct Data {
-    map: HashMap<server::Server, HashMap<history::Kind, History>>,
+    map: HashMap<history::Kind, History>,
     input: input::Storage,
 }
 
 impl Data {
-    fn loaded(
-        &mut self,
-        server: server::Server,
-        kind: history::Kind,
-        mut messages: Vec<crate::Message>,
-    ) {
+    fn load_full(&mut self, kind: history::Kind, data: history::Loaded) {
         use std::collections::hash_map;
 
-        match self
-            .map
-            .entry(server.clone())
-            .or_default()
-            .entry(kind.clone())
-        {
+        let history::Loaded {
+            mut messages,
+            metadata,
+        } = data;
+
+        match self.map.entry(kind.clone()) {
             hash_map::Entry::Occupied(mut entry) => match entry.get_mut() {
                 History::Partial {
                     messages: new_messages,
-                    last_received_at,
-                    opened_at,
+                    last_updated_at,
+                    read_marker: partial_read_marker,
                     ..
                 } => {
-                    let last_received_at = *last_received_at;
-                    let opened_at = *opened_at;
+                    let read_marker = (*partial_read_marker).max(metadata.read_marker);
+
+                    let last_updated_at = *last_updated_at;
                     messages.extend(std::mem::take(new_messages));
                     entry.insert(History::Full {
-                        server,
                         kind,
                         messages,
-                        last_received_at,
-                        opened_at,
+                        last_updated_at,
+                        read_marker,
                     });
                 }
                 _ => {
                     entry.insert(History::Full {
-                        server,
                         kind,
                         messages,
-                        last_received_at: None,
-                        opened_at: Posix::now(),
+                        last_updated_at: None,
+                        read_marker: metadata.read_marker,
                     });
                 }
             },
             hash_map::Entry::Vacant(entry) => {
                 entry.insert(History::Full {
-                    server,
                     kind,
                     messages,
-                    last_received_at: None,
-                    opened_at: Posix::now(),
+                    last_updated_at: None,
+                    read_marker: metadata.read_marker,
                 });
             }
         }
     }
 
+    fn update_partial(&mut self, kind: history::Kind, data: history::Metadata) {
+        if let Some(history) = self.map.get_mut(&kind) {
+            history.update_partial(data);
+        }
+    }
+
     fn history_view(
         &self,
-        server: &server::Server,
         kind: &history::Kind,
         limit: Option<Limit>,
         buffer_config: &config::Buffer,
     ) -> Option<history::View> {
         let History::Full {
             messages,
-            opened_at,
+            read_marker,
             ..
-        } = self.map.get(server)?.get(kind)?
+        } = self.map.get(kind)?
         else {
             return None;
         };
@@ -579,7 +599,7 @@ impl Data {
         });
 
         let max_prefix_chars = buffer_config.nickname.alignment.is_right().then(|| {
-            if matches!(kind, history::Kind::Channel(_)) {
+            if matches!(kind, history::Kind::Channel(..)) {
                 filtered
                     .iter()
                     .filter_map(|message| {
@@ -600,12 +620,33 @@ impl Data {
             }
         });
 
+        let has_read_messages = read_marker
+            .map(|marker| {
+                filtered
+                    .iter()
+                    .any(|message| message.server_time <= marker.date_time())
+            })
+            .unwrap_or_default();
+
         let limited = with_limit(limit, filtered.into_iter());
 
-        let split_at = limited
-            .iter()
-            .position(|message| message.received_at >= *opened_at)
-            .unwrap_or(limited.len());
+        let split_at = read_marker.map_or(0, |read_marker| {
+            limited
+                .iter()
+                .rev()
+                .position(|message| message.server_time <= read_marker.date_time())
+                .map_or_else(
+                    || {
+                        // Backlog is before this limit view of messages
+                        if has_read_messages {
+                            0
+                        } else {
+                            limited.len()
+                        }
+                    },
+                    |position| limited.len() - position,
+                )
+        });
 
         let (old, new) = limited.split_at(split_at);
 
@@ -620,40 +661,102 @@ impl Data {
 
     fn add_message(
         &mut self,
-        server: server::Server,
         kind: history::Kind,
         message: crate::Message,
-    ) {
-        self.map
-            .entry(server.clone())
-            .or_default()
-            .entry(kind.clone())
-            .or_insert_with(|| History::partial(server, kind, message.received_at))
-            .add_message(message)
+    ) -> Option<impl Future<Output = Message>> {
+        use std::collections::hash_map;
+
+        match self.map.entry(kind.clone()) {
+            hash_map::Entry::Occupied(mut entry) => {
+                entry.get_mut().add_message(message);
+
+                None
+            }
+            hash_map::Entry::Vacant(entry) => {
+                entry
+                    .insert(History::partial(kind.clone()))
+                    .add_message(message);
+
+                Some(
+                    async move {
+                        let loaded = history::metadata::load(kind.clone()).await;
+
+                        Message::UpdatePartial(kind, loaded)
+                    }
+                    .boxed(),
+                )
+            }
+        }
+    }
+
+    fn update_read_marker(
+        &mut self,
+        kind: impl Into<history::Kind>,
+        read_marker: history::ReadMarker,
+    ) -> Option<impl Future<Output = Message>> {
+        use std::collections::hash_map;
+
+        let kind = kind.into();
+
+        match self.map.entry(kind.clone()) {
+            hash_map::Entry::Occupied(mut entry) => {
+                entry.get_mut().update_read_marker(read_marker);
+
+                None
+            }
+            hash_map::Entry::Vacant(_) => Some(
+                async move {
+                    let updated = history::metadata::update(&kind, &read_marker).await;
+
+                    Message::UpdateReadMarker(kind, read_marker, updated)
+                }
+                .boxed(),
+            ),
+        }
+    }
+
+    fn channel_joined(
+        &mut self,
+        server: server::Server,
+        channel: String,
+    ) -> Option<impl Future<Output = Message>> {
+        use std::collections::hash_map;
+
+        let kind = history::Kind::Channel(server, channel);
+
+        match self.map.entry(kind.clone()) {
+            hash_map::Entry::Occupied(_) => None,
+            hash_map::Entry::Vacant(entry) => {
+                entry.insert(History::partial(kind.clone()));
+
+                Some(
+                    async move {
+                        let loaded = history::metadata::load(kind.clone()).await;
+
+                        Message::UpdatePartial(kind, loaded)
+                    }
+                    .boxed(),
+                )
+            }
+        }
     }
 
     fn untrack(
         &mut self,
-        server: &server::Server,
         kind: &history::Kind,
-    ) -> Option<impl Future<Output = Result<(), history::Error>>> {
-        self.map
-            .get_mut(server)
-            .and_then(|map| map.get_mut(kind).and_then(History::make_partial))
+    ) -> Option<impl Future<Output = Result<Option<history::ReadMarker>, history::Error>>> {
+        self.map.get_mut(kind).and_then(History::make_partial)
     }
 
     fn flush_all(&mut self, now: Instant) -> Vec<BoxFuture<'static, Message>> {
         self.map
             .iter_mut()
-            .flat_map(|(server, map)| {
-                map.iter_mut().filter_map(|(kind, state)| {
-                    let server = server.clone();
-                    let kind = kind.clone();
+            .filter_map(|(kind, state)| {
+                let kind = kind.clone();
 
-                    state.flush(now).map(move |task| {
-                        task.map(move |result| Message::Flushed(server, kind, result))
-                            .boxed()
-                    })
+                state.flush(now).map(move |task| {
+                    task.map(move |result| Message::Flushed(kind, result))
+                        .boxed()
                 })
             })
             .collect()
