@@ -10,12 +10,13 @@ pub use self::mode::Mode;
 pub use self::pane::Pane;
 pub use self::server::Server;
 pub use self::shortcut::Shortcut;
-pub use self::theme::Theme;
+pub use self::appearance::Theme;
 pub use self::url::Url;
 pub use self::user::User;
 pub use self::version::Version;
 pub use self::window::Window;
 
+pub mod appearance;
 pub mod audio;
 pub mod buffer;
 pub mod channel;
@@ -38,7 +39,6 @@ pub mod pane;
 pub mod server;
 pub mod shortcut;
 pub mod stream;
-pub mod theme;
 pub mod time;
 pub mod url;
 pub mod user;
