@@ -1,4 +1,5 @@
 use std::borrow::Cow;
+use std::hash::{DefaultHasher, Hash as _, Hasher};
 use std::iter;
 
 use chrono::{DateTime, Utc};
@@ -17,7 +18,7 @@ pub use self::source::Source;
 use crate::config::buffer::UsernameFormat;
 use crate::time::{self, Posix};
 use crate::user::{Nick, NickRef};
-use crate::{ctcp, Config, User};
+use crate::{ctcp, Config, Server, User};
 
 // References:
 // - https://datatracker.ietf.org/doc/html/rfc1738#section-5
@@ -116,6 +117,12 @@ pub enum Target {
         nick: Nick,
         source: Source,
     },
+    Logs,
+    Highlights {
+        server: Server,
+        channel: Channel,
+        source: Source,
+    },
 }
 
 impl Target {
@@ -124,6 +131,8 @@ impl Target {
             Target::Server { .. } => None,
             Target::Channel { prefix, .. } => prefix.as_ref(),
             Target::Query { .. } => None,
+            Target::Logs => None,
+            Target::Highlights { .. } => None,
         }
     }
 
@@ -132,6 +141,8 @@ impl Target {
             Target::Server { source } => source,
             Target::Channel { source, .. } => source,
             Target::Query { source, .. } => source,
+            Target::Logs => &Source::Internal(source::Internal::Logs),
+            Target::Highlights { source, .. } => source,
         }
     }
 }
@@ -150,6 +161,7 @@ pub struct Message {
     pub target: Target,
     pub content: Content,
     pub id: Option<String>,
+    pub hash: Hash,
 }
 
 impl Message {
@@ -165,6 +177,7 @@ impl Message {
                             | source::server::Kind::MonitoredOffline
                     )
                 }
+                Source::Internal(source::Internal::Logs) => true,
                 _ => false,
             }
     }
@@ -186,42 +199,70 @@ impl Message {
             &channel_users,
         )?;
         let target = target(encoded, &our_nick, &resolve_attributes)?;
+        let received_at = Posix::now();
+        let hash = Hash::new(&received_at, &content);
 
         Some(Message {
-            received_at: Posix::now(),
+            received_at,
             server_time,
             direction: Direction::Received,
             target,
             content,
             id,
+            hash,
         })
     }
 
-    pub fn file_transfer_request_received(from: &Nick, filename: &str) -> Message {
+    pub fn sent(target: Target, content: Content) -> Self {
+        let received_at = Posix::now();
+        let hash = Hash::new(&received_at, &content);
+
         Message {
-            received_at: Posix::now(),
+            received_at,
+            server_time: Utc::now(),
+            direction: Direction::Sent,
+            target,
+            content,
+            id: None,
+            hash,
+        }
+    }
+
+    pub fn file_transfer_request_received(from: &Nick, filename: &str) -> Message {
+        let received_at = Posix::now();
+        let content = plain(format!("{from} wants to send you \"{filename}\""));
+        let hash = Hash::new(&received_at, &content);
+
+        Message {
+            received_at,
             server_time: Utc::now(),
             direction: Direction::Received,
             target: Target::Query {
                 nick: from.clone(),
                 source: Source::Action,
             },
-            content: plain(format!("{from} wants to send you \"{filename}\"")),
+            content,
             id: None,
+            hash,
         }
     }
 
     pub fn file_transfer_request_sent(to: &Nick, filename: &str) -> Message {
+        let received_at = Posix::now();
+        let content = plain(format!("offering to send {to} \"{filename}\""));
+        let hash = Hash::new(&received_at, &content);
+
         Message {
-            received_at: Posix::now(),
+            received_at,
             server_time: Utc::now(),
             direction: Direction::Sent,
             target: Target::Query {
                 nick: to.clone(),
                 source: Source::Action,
             },
-            content: plain(format!("offering to send {to} \"{filename}\"")),
+            content,
             id: None,
+            hash,
         }
     }
 
@@ -233,8 +274,43 @@ impl Message {
         match &self.content {
             Content::Plain(s) => Some(s),
             Content::Fragments(_) => None,
+            Content::Log(_) => None,
         }
     }
+
+    pub fn log(record: crate::log::Record) -> Self {
+        let received_at = Posix::now();
+        let server_time = record.timestamp;
+        let content = Content::Log(record);
+        let hash = Hash::new(&received_at, &content);
+
+        Self {
+            received_at,
+            server_time,
+            direction: Direction::Received,
+            target: Target::Logs,
+            content,
+            id: None,
+            hash,
+        }
+    }
+
+    pub fn into_highlight(mut self, server: Server) -> Option<Self> {
+        self.target = match self.target {
+            Target::Channel {
+                channel,
+                source: Source::User(user),
+                ..
+            } => Target::Highlights {
+                server,
+                channel,
+                source: Source::User(user),
+            },
+            _ => return None,
+        };
+
+        Some(self)
+    }
 }
 
 impl Serialize for Message {
@@ -305,6 +381,8 @@ impl<'de> Deserialize<'de> for Message {
             Content::Plain("".to_string())
         };
 
+        let hash = Hash::new(&received_at, &content);
+
         Ok(Message {
             received_at,
             server_time,
@@ -312,10 +390,23 @@ impl<'de> Deserialize<'de> for Message {
             target,
             content,
             id,
+            hash,
         })
     }
 }
 
+#[derive(Debug, Clone, Copy, PartialEq, Eq)]
+pub struct Hash(u64);
+
+impl Hash {
+    pub fn new(received_at: &time::Posix, content: &Content) -> Self {
+        let mut hasher = DefaultHasher::new();
+        received_at.hash(&mut hasher);
+        content.hash(&mut hasher);
+        Self(hasher.finish())
+    }
+}
+
 pub fn plain(text: String) -> Content {
     Content::Plain(text)
 }
@@ -383,30 +474,37 @@ fn parse_url_fragments(text: String) -> Vec<Fragment> {
     fragments
 }
 
+/// Checks if a given `text` contains or matches a user's nickname.
+fn text_references_nickname(text: &str, nickname: NickRef) -> Option<bool> {
+    // TODO: Consider server case-mapping settings vs just ascii lowercase
+    let nick = nickname.as_ref();
+    let nick_lower = nick.to_ascii_lowercase();
+    let lower = text.to_ascii_lowercase();
+    let trimmed = text.trim_matches(|c: char| c.is_ascii_punctuation());
+    let lower_trimmed = trimmed.to_ascii_lowercase();
+
+    if nick == text || nick_lower == lower {
+        // Contains the user's nickname without trimming.
+        Some(false)
+    } else if nick == trimmed || nick_lower == lower_trimmed {
+        // Contains the user's nickname with trimming.
+        Some(true)
+    } else {
+        // Doesn't contain the user's nickname.
+        None
+    }
+}
+
 fn parse_user_and_channel_fragments(text: &str, channel_users: &[User]) -> Vec<Fragment> {
     text.chars()
         .group_by(|c| c.is_whitespace())
         .into_iter()
         .flat_map(|(is_whitespace, chars)| {
             let text = chars.collect::<String>();
-            let trimmed = text.trim_matches(|c: char| c.is_ascii_punctuation());
-            let lower = text.to_ascii_lowercase();
-            let lower_trimmed = trimmed.to_ascii_lowercase();
-
             if !is_whitespace {
                 if let Some((is_trimmed, user)) = channel_users.iter().find_map(|user| {
-                    let nickname = user.nickname();
-                    let nick = nickname.as_ref();
-                    // TODO: Consider server case-mapping settings vs just ascii lowercase
-                    let nick_lower = nick.to_ascii_lowercase();
-
-                    if nick == text || nick_lower == lower {
-                        Some((false, user.clone()))
-                    } else if nick == trimmed || nick_lower == lower_trimmed {
-                        Some((true, user.clone()))
-                    } else {
-                        None
-                    }
+                    text_references_nickname(text.as_str(), user.nickname())
+                        .map(|is_trimmed| (is_trimmed, user.clone()))
                 }) {
                     if is_trimmed {
                         let prefix_end = text.find(|c: char| !c.is_ascii_punctuation());
@@ -456,10 +554,11 @@ fn parse_user_and_channel_fragments(text: &str, channel_users: &[User]) -> Vec<F
         })
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum Content {
     Plain(String),
     Fragments(Vec<Fragment>),
+    Log(crate::log::Record),
 }
 
 impl Content {
@@ -467,11 +566,12 @@ impl Content {
         match self {
             Content::Plain(s) => s.into(),
             Content::Fragments(fragments) => fragments.iter().map(Fragment::as_str).join("").into(),
+            Content::Log(record) => (&record.message).into(),
         }
     }
 }
 
-#[derive(Debug, Clone, PartialEq, Eq, Serialize, Deserialize)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Serialize, Deserialize)]
 pub enum Fragment {
     Text(String),
     Channel(String),
@@ -694,6 +794,7 @@ fn target(
         | Command::CNOTICE(_, _, _)
         | Command::CPRIVMSG(_, _, _)
         | Command::KNOCK(_, _)
+        | Command::MARKREAD(_, _)
         | Command::MONITOR(_, _)
         | Command::TAGMSG(_)
         | Command::USERIP(_)
@@ -1011,7 +1112,7 @@ pub enum Limit {
 
 impl Limit {
     pub const DEFAULT_STEP: usize = 50;
-    const DEFAULT_COUNT: usize = 500;
+    pub const DEFAULT_COUNT: usize = 500;
 
     pub fn top() -> Self {
         Self::Top(Self::DEFAULT_COUNT)
@@ -1059,19 +1160,30 @@ fn monitored_targets_text(targets: Vec<String>) -> Option<String> {
     }
 }
 
-pub fn reference_user(sender: NickRef, own_nick: NickRef, message: &Message) -> bool {
-    let has_nick = match &message.content {
-        Content::Plain(text) => text.contains(own_nick.as_ref()),
+pub fn references_user(sender: NickRef, own_nick: NickRef, message: &Message) -> bool {
+    match &message.content {
+        Content::Plain(text) => references_user_text(sender, own_nick, text),
         Content::Fragments(fragments) => fragments
             .iter()
-            .any(|f| f.as_str().contains(own_nick.as_ref())),
-    };
-
-    sender != own_nick && has_nick
+            .any(|f| references_user_text(sender, own_nick, f.as_str())),
+        Content::Log(_) => false,
+    }
 }
 
-pub fn reference_user_text(sender: NickRef, own_nick: NickRef, text: &str) -> bool {
-    sender != own_nick && text.contains(own_nick.as_ref())
+pub fn references_user_text(sender: NickRef, own_nick: NickRef, text: &str) -> bool {
+    sender != own_nick
+        && text
+            .chars()
+            .group_by(|c| c.is_whitespace())
+            .into_iter()
+            .any(|(is_whitespace, chars)| {
+                if !is_whitespace {
+                    let text = chars.collect::<String>();
+                    text_references_nickname(&text, own_nick).is_some()
+                } else {
+                    false
+                }
+            })
 }
 
 #[derive(Debug, Clone)]
@@ -1079,6 +1191,7 @@ pub enum Link {
     Channel(String),
     Url(String),
     User(User),
+    GoToMessage(Server, String, Hash),
 }
 
 fn fail_as_none<'de, T, D>(deserializer: D) -> Result<Option<T>, D::Error>
