@@ -1,9 +1,10 @@
+use chrono::{DateTime, Utc};
+use futures::never::Never;
 use std::time::Duration;
 
 use futures::channel::mpsc;
-use futures::never::Never;
-use futures::{stream, FutureExt, SinkExt, StreamExt};
-use irc::proto::{self, command};
+use futures::{future, stream, FutureExt, SinkExt, StreamExt};
+use irc::proto::{self, command, Command};
 use irc::{codec, connection, Connection};
 use tokio::time::{self, Instant, Interval};
 
@@ -25,17 +26,21 @@ pub enum Update {
         server: Server,
         client: Client,
         is_initial: bool,
+        sent_time: DateTime<Utc>,
     },
     Disconnected {
         server: Server,
         is_initial: bool,
         error: Option<String>,
+        sent_time: DateTime<Utc>,
     },
     ConnectionFailed {
         server: Server,
         error: String,
+        sent_time: DateTime<Utc>,
     },
     MessagesReceived(Server, Vec<message::Encoded>),
+    Quit(Server, Option<String>),
 }
 
 enum State {
@@ -48,6 +53,7 @@ enum State {
         ping_time: Interval,
         ping_timeout: Option<Interval>,
     },
+    Quit,
 }
 
 enum Input {
@@ -59,11 +65,28 @@ enum Input {
 }
 
 struct Stream {
-    connection: Connection,
+    connection: Connection<irc::Codec>,
     receiver: mpsc::Receiver<proto::Message>,
 }
 
-pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Never {
+pub fn run(
+    server: server::Entry,
+    proxy: Option<config::Proxy>,
+) -> impl futures::Stream<Item = Update> {
+    let (sender, receiver) = mpsc::unbounded();
+
+    // Spawn to unblock backend from iced stream which has backpressure
+    let runner = stream::once(async { tokio::spawn(_run(server, proxy, sender)).await })
+        .map(|_| unreachable!());
+
+    stream::select(receiver, runner)
+}
+
+async fn _run(
+    server: server::Entry,
+    proxy: Option<config::Proxy>,
+    sender: mpsc::UnboundedSender<Update>,
+) -> Never {
     let server::Entry { server, config } = server;
 
     let reconnect_delay = Duration::from_secs(config.reconnect_delay);
@@ -72,13 +95,12 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
     let mut state = State::Disconnected { last_retry: None };
 
     // Notify app of initial disconnected state
-    let _ = sender
-        .send(Update::Disconnected {
-            server: server.clone(),
-            is_initial,
-            error: None,
-        })
-        .await;
+    let _ = sender.unbounded_send(Update::Disconnected {
+        server: server.clone(),
+        is_initial,
+        error: None,
+        sent_time: Utc::now(),
+    });
 
     loop {
         match &mut state {
@@ -91,17 +113,16 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
                     }
                 }
 
-                match connect(server.clone(), config.clone()).await {
+                match connect(server.clone(), config.clone(), proxy.clone()).await {
                     Ok((stream, client)) => {
                         log::info!("[{server}] connected");
 
-                        let _ = sender
-                            .send(Update::Connected {
-                                server: server.clone(),
-                                client,
-                                is_initial,
-                            })
-                            .await;
+                        let _ = sender.unbounded_send(Update::Connected {
+                            server: server.clone(),
+                            client,
+                            is_initial,
+                            sent_time: Utc::now(),
+                        });
 
                         is_initial = false;
 
@@ -121,12 +142,11 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
 
                         log::warn!("[{server}] connection failed: {error}");
 
-                        let _ = sender
-                            .send(Update::ConnectionFailed {
-                                server: server.clone(),
-                                error,
-                            })
-                            .await;
+                        let _ = sender.unbounded_send(Update::ConnectionFailed {
+                            server: server.clone(),
+                            error,
+                            sent_time: Utc::now(),
+                        });
 
                         *last_retry = Some(Instant::now());
                     }
@@ -172,13 +192,12 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
                         }
                         proto::Command::ERROR(error) => {
                             log::warn!("[{server}] disconnected: {error}");
-                            let _ = sender
-                                .send(Update::Disconnected {
-                                    server: server.clone(),
-                                    is_initial,
-                                    error: Some(error),
-                                })
-                                .await;
+                            let _ = sender.unbounded_send(Update::Disconnected {
+                                server: server.clone(),
+                                is_initial,
+                                error: Some(error),
+                                sent_time: Utc::now(),
+                            });
                             state = State::Disconnected {
                                 last_retry: Some(Instant::now()),
                             };
@@ -192,24 +211,33 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
                     }
                     Input::IrcMessage(Err(e)) => {
                         log::warn!("[{server}] disconnected: {e}");
-                        let _ = sender
-                            .send(Update::Disconnected {
-                                server: server.clone(),
-                                is_initial,
-                                error: Some(e.to_string()),
-                            })
-                            .await;
+                        let _ = sender.unbounded_send(Update::Disconnected {
+                            server: server.clone(),
+                            is_initial,
+                            error: Some(e.to_string()),
+                            sent_time: Utc::now(),
+                        });
                         state = State::Disconnected {
                             last_retry: Some(Instant::now()),
                         };
                     }
                     Input::Batch(messages) => {
                         let _ = sender
-                            .send(Update::MessagesReceived(server.clone(), messages))
-                            .await;
+                            .unbounded_send(Update::MessagesReceived(server.clone(), messages));
                     }
                     Input::Send(message) => {
-                        let _ = stream.connection.send(message).await;
+                        if let Command::QUIT(reason) = &message.command {
+                            let reason = reason.clone();
+
+                            let _ = stream.connection.send(message).await;
+                            let _ = sender.unbounded_send(Update::Quit(server.clone(), reason));
+
+                            log::info!("[{server}] quit");
+
+                            state = State::Quit;
+                        } else {
+                            let _ = stream.connection.send(message).await;
+                        }
                     }
                     Input::Ping => {
                         let now = Posix::now().as_nanos().to_string();
@@ -223,19 +251,22 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
                     }
                     Input::PingTimeout => {
                         log::warn!("[{server}] ping timeout");
-                        let _ = sender
-                            .send(Update::Disconnected {
-                                server: server.clone(),
-                                is_initial,
-                                error: Some("ping timeout".into()),
-                            })
-                            .await;
+                        let _ = sender.unbounded_send(Update::Disconnected {
+                            server: server.clone(),
+                            is_initial,
+                            error: Some("ping timeout".into()),
+                            sent_time: Utc::now(),
+                        });
                         state = State::Disconnected {
                             last_retry: Some(Instant::now()),
                         };
                     }
                 }
             }
+            State::Quit => {
+                // Wait forever until this stream is dropped by the frontend
+                future::pending::<()>().await;
+            }
         }
     }
 }
@@ -243,8 +274,9 @@ pub async fn run(server: server::Entry, mut sender: mpsc::Sender<Update>) -> Nev
 async fn connect(
     server: Server,
     config: config::Server,
+    proxy: Option<config::Proxy>,
 ) -> Result<(Stream, Client), connection::Error> {
-    let connection = Connection::new(config.connection()).await?;
+    let connection = Connection::new(config.connection(proxy), irc::Codec).await?;
 
     let (sender, receiver) = mpsc::channel(100);
 
