@@ -1,6 +1,6 @@
 use data::input::{self, Cache, Draft};
 use data::user::Nick;
-use data::{client, history, Buffer, Config};
+use data::{buffer, client, history, Config};
 use iced::widget::{container, row, text, text_input};
 use iced::Task;
 
@@ -11,7 +11,9 @@ use crate::widget::{anchored_overlay, key_press, Element};
 mod completion;
 
 pub enum Event {
-    InputSent,
+    InputSent {
+        history_task: Task<history::manager::Message>,
+    },
 }
 
 #[derive(Debug, Clone)]
@@ -117,7 +119,7 @@ impl State {
     pub fn update(
         &mut self,
         message: Message,
-        buffer: &Buffer,
+        buffer: &buffer::Upstream,
         clients: &mut client::Map,
         history: &mut history::Manager,
         config: &Config,
@@ -177,12 +179,14 @@ impl State {
                         clients.send(buffer, encoded);
                     }
 
+                    let mut history_task = Task::none();
+
                     if let Some(nick) = clients.nickname(buffer.server()) {
                         let mut user = nick.to_owned().into();
                         let mut channel_users = &[][..];
 
                         // Resolve our attributes if sending this message in a channel
-                        if let Buffer::Channel(server, channel) = &buffer {
+                        if let buffer::Upstream::Channel(server, channel) = &buffer {
                             channel_users = clients.get_channel_users(server, channel);
 
                             if let Some(user_with_attributes) =
@@ -192,10 +196,15 @@ impl State {
                             }
                         }
 
-                        history.record_input(input, user, channel_users);
+                        history_task = Task::batch(
+                            history
+                                .record_input(input, user, channel_users)
+                                .into_iter()
+                                .map(Task::future),
+                        );
                     }
 
-                    (Task::none(), Some(Event::InputSent))
+                    (Task::none(), Some(Event::InputSent { history_task }))
                 } else {
                     (Task::none(), None)
                 }
@@ -279,7 +288,7 @@ impl State {
 
     fn on_completion(
         &self,
-        buffer: &Buffer,
+        buffer: &buffer::Upstream,
         history: &mut history::Manager,
         text: String,
     ) -> (Task<Message>, Option<Event>) {
@@ -304,7 +313,7 @@ impl State {
     pub fn insert_user(
         &mut self,
         nick: Nick,
-        buffer: Buffer,
+        buffer: buffer::Upstream,
         history: &mut history::Manager,
     ) -> Task<Message> {
         let mut text = history.input(&buffer).draft.to_string();
