@@ -1,4 +1,4 @@
-use data::{history, message, Config};
+use data::{buffer, history, message, Config};
 use iced::widget::{column, container, row, vertical_space};
 use iced::{Length, Task};
 
@@ -12,10 +12,10 @@ pub enum Message {
     InputView(input_view::Message),
 }
 
-#[derive(Debug, Clone)]
 pub enum Event {
     UserContext(user_context::Event),
     OpenChannel(String),
+    History(Task<history::manager::Message>),
 }
 
 pub fn view<'a>(
@@ -103,7 +103,7 @@ pub fn view<'a>(
 
 #[derive(Debug, Clone)]
 pub struct Server {
-    pub buffer: data::Buffer,
+    pub buffer: buffer::Upstream,
     pub server: data::server::Server,
     pub scroll_view: scroll_view::State,
     pub input_view: input_view::State,
@@ -112,7 +112,7 @@ pub struct Server {
 impl Server {
     pub fn new(server: data::server::Server) -> Self {
         Self {
-            buffer: data::Buffer::Server(server.clone()),
+            buffer: buffer::Upstream::Server(server.clone()),
             server,
             scroll_view: scroll_view::State::new(),
             input_view: input_view::State::new(),
@@ -130,9 +130,10 @@ impl Server {
             Message::ScrollView(message) => {
                 let (command, event) = self.scroll_view.update(message);
 
-                let event = event.map(|event| match event {
-                    scroll_view::Event::UserContext(event) => Event::UserContext(event),
-                    scroll_view::Event::OpenChannel(channel) => Event::OpenChannel(channel),
+                let event = event.and_then(|event| match event {
+                    scroll_view::Event::UserContext(event) => Some(Event::UserContext(event)),
+                    scroll_view::Event::OpenChannel(channel) => Some(Event::OpenChannel(channel)),
+                    scroll_view::Event::GoToMessage(_, _, _) => None,
                 });
 
                 (command.map(Message::ScrollView), event)
@@ -143,15 +144,16 @@ impl Server {
                         .update(message, &self.buffer, clients, history, config);
                 let command = command.map(Message::InputView);
 
-                let task = match event {
-                    Some(input_view::Event::InputSent) => Task::batch(vec![
-                        command,
-                        self.scroll_view.scroll_to_end().map(Message::ScrollView),
-                    ]),
-                    None => command,
-                };
-
-                (task, None)
+                match event {
+                    Some(input_view::Event::InputSent { history_task }) => (
+                        Task::batch(vec![
+                            command,
+                            self.scroll_view.scroll_to_end().map(Message::ScrollView),
+                        ]),
+                        Some(Event::History(history_task)),
+                    ),
+                    None => (command, None),
+                }
             }
         }
     }
