@@ -72,6 +72,26 @@ pub fn popout<'a>() -> Text<'a> {
     to_text('\u{E80E}')
 }
 
+pub fn logs<'a>() -> Text<'a> {
+    to_text('\u{E810}')
+}
+
+pub fn menu<'a>() -> Text<'a> {
+    to_text('\u{F0C9}')
+}
+
+pub fn documentation<'a>() -> Text<'a> {
+    to_text('\u{E812}')
+}
+
+pub fn highlights<'a>() -> Text<'a> {
+    to_text('\u{E811}')
+}
+
+pub fn scroll_to_bottom<'a>() -> Text<'a> {
+    to_text('\u{E814}')
+}
+
 fn to_text<'a>(unicode: char) -> Text<'a> {
     text(unicode.to_string())
         .line_height(LineHeight::Relative(1.0))
