@@ -1,6 +1,7 @@
 #![allow(clippy::large_enum_variant, clippy::too_many_arguments)]
 #![cfg_attr(not(debug_assertions), windows_subsystem = "windows")]
 
+mod appearance;
 mod audio;
 mod buffer;
 mod event;
@@ -11,26 +12,29 @@ mod modal;
 mod notification;
 mod screen;
 mod stream;
-mod theme;
 mod url;
 mod widget;
 mod window;
 
-use std::env;
+use std::collections::HashSet;
 use std::time::{Duration, Instant};
+use std::{env, mem};
 
+use appearance::{theme, Theme};
 use chrono::Utc;
 use data::config::{self, Config};
+use data::history::manager::Broadcast;
 use data::version::Version;
-use data::{environment, history, server, version, Url, User};
+use data::{environment, server, version, Url, User};
+use data::{history, Server};
 use iced::widget::{column, container};
 use iced::{padding, Length, Subscription, Task};
 use screen::{dashboard, help, migration, welcome};
 use tokio::runtime;
+use tokio_stream::wrappers::ReceiverStream;
 
 use self::event::{events, Event};
 use self::modal::Modal;
-use self::theme::Theme;
 use self::widget::Element;
 use self::window::Window;
 
@@ -54,7 +58,7 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
     // Prepare notifications.
     notification::prepare();
 
-    logger::setup(is_debug).expect("setup logging");
+    let log_stream = logger::setup(is_debug).expect("setup logging");
     log::info!("halloy {} has started", environment::formatted_version());
     log::info!("config dir: {:?}", environment::config_dir());
     log::info!("data dir: {:?}", environment::data_dir());
@@ -91,7 +95,7 @@ pub fn main() -> Result<(), Box<dyn std::error::Error>> {
         .scale_factor(Halloy::scale_factor)
         .subscription(Halloy::subscription)
         .settings(settings(&config_load))
-        .run_with(move || Halloy::new(config_load.clone(), destination.clone()))
+        .run_with(move || Halloy::new(config_load, destination, log_stream))
         .inspect_err(|err| log::error!("{}", err))?;
 
     Ok(())
@@ -123,6 +127,7 @@ struct Halloy {
     servers: server::Map,
     modal: Option<Modal>,
     main_window: Window,
+    pending_logs: Vec<data::log::Record>,
 }
 
 impl Halloy {
@@ -151,43 +156,41 @@ impl Halloy {
                     command.map(Message::Dashboard),
                 )
             }
-            Err(error) => match &error {
-                config::Error::Parse(_) | config::Error::LoadSounds(_) => (
-                    Screen::Help(screen::Help::new(error)),
-                    Config::default(),
-                    Task::none(),
-                ),
-                _ => {
-                    // If we have a YAML file, but end up in this arm
-                    // it means the user tried to load Halloy with a YAML configuration, but it expected TOML.
-                    if config::has_yaml_config() {
-                        (
-                            Screen::Migration(screen::Migration::new()),
-                            Config::default(),
-                            Task::none(),
-                        )
-                    } else {
-                        // Otherwise, show regular welcome screen for new users.
-                        (
-                            Screen::Welcome(screen::Welcome::new()),
-                            Config::default(),
-                            Task::none(),
-                        )
-                    }
-                }
-            },
+            // If we have a YAML file, but end up in this arm
+            // it means the user tried to load Halloy with a YAML configuration, but it expected TOML.
+            Err(config::Error::ConfigMissing {
+                has_yaml_config: true,
+            }) => (
+                Screen::Migration(screen::Migration::new()),
+                Config::default(),
+                Task::none(),
+            ),
+            // Show regular welcome screen for new users.
+            Err(config::Error::ConfigMissing {
+                has_yaml_config: false,
+            }) => (
+                Screen::Welcome(screen::Welcome::new()),
+                Config::default(),
+                Task::none(),
+            ),
+            Err(error) => (
+                Screen::Help(screen::Help::new(error)),
+                Config::default(),
+                Task::none(),
+            ),
         };
 
         (
             Halloy {
                 version: Version::new(),
                 screen,
-                theme: config.themes.default.clone().into(),
+                theme: appearance::theme(&config.appearance.selected).into(),
                 clients: Default::default(),
                 servers: config.servers.clone(),
                 config,
                 modal: None,
                 main_window,
+                pending_logs: vec![],
             },
             command,
         )
@@ -199,11 +202,12 @@ pub enum Screen {
     Help(screen::Help),
     Welcome(screen::Welcome),
     Migration(screen::Migration),
+    Exit { pending_exit: HashSet<Server> },
 }
 
 #[derive(Debug)]
 pub enum Message {
-    ThemesReloaded(config::Themes),
+    AppearanceReloaded(data::appearance::Appearance),
     ScreenConfigReloaded(Result<Config, config::Error>),
     Dashboard(dashboard::Message),
     Stream(stream::Update),
@@ -215,14 +219,17 @@ pub enum Message {
     Version(Option<String>),
     Modal(modal::Message),
     RouteReceived(String),
+    AppearanceChange(appearance::Mode),
     Window(window::Id, window::Event),
     WindowSettingsSaved(Result<(), window::Error>),
+    Logging(Vec<logger::Record>),
 }
 
 impl Halloy {
     fn new(
         config_load: Result<Config, config::Error>,
         url_received: Option<data::Url>,
+        log_stream: ReceiverStream<Vec<logger::Record>>,
     ) -> (Halloy, Task<Message>) {
         let (main_window, open_main_window) = window::open(window::Settings {
             size: window::default_size(),
@@ -240,6 +247,7 @@ impl Halloy {
             open_main_window.then(|_| Task::none()),
             command,
             latest_remote_version,
+            Task::stream(log_stream).map(Message::Logging),
         ];
 
         if let Some(url) = url_received {
@@ -279,8 +287,8 @@ impl Halloy {
 
     fn update(&mut self, message: Message) -> Task<Message> {
         match message {
-            Message::ThemesReloaded(updated) => {
-                self.config.themes = updated;
+            Message::AppearanceReloaded(appearance) => {
+                self.config.appearance = appearance;
                 Task::none()
             }
             Message::ScreenConfigReloaded(updated) => {
@@ -317,7 +325,7 @@ impl Halloy {
                                     .collect::<Vec<_>>();
 
                                 self.servers = updated.servers.clone();
-                                self.theme = updated.themes.default.clone().into();
+                                self.theme = appearance::theme(&updated.appearance.selected).into();
                                 self.config = updated;
 
                                 for server in removed_servers {
@@ -331,12 +339,22 @@ impl Halloy {
                         Task::none()
                     }
                     Some(dashboard::Event::ReloadThemes) => Task::future(Config::load())
-                        .and_then(|config| Task::done(config.themes))
-                        .map(Message::ThemesReloaded),
+                        .and_then(|config| Task::done(config.appearance))
+                        .map(Message::AppearanceReloaded),
                     Some(dashboard::Event::QuitServer(server)) => {
                         self.clients.quit(&server, None);
                         Task::none()
                     }
+                    Some(dashboard::Event::Exit) => {
+                        let pending_exit = self.clients.exit();
+
+                        if pending_exit.is_empty() {
+                            iced::exit()
+                        } else {
+                            self.screen = Screen::Exit { pending_exit };
+                            Task::none()
+                        }
+                    }
                     None => Task::none(),
                 };
 
@@ -403,14 +421,21 @@ impl Halloy {
 
                     if is_initial {
                         // Intial is sent when first trying to connect
-                        dashboard.broadcast_connecting(&server, &self.config, sent_time);
+                        dashboard
+                            .broadcast(&server, &self.config, sent_time, Broadcast::Connecting)
+                            .map(Message::Dashboard)
                     } else {
                         notification::disconnected(&self.config.notifications, &server);
 
-                        dashboard.broadcast_disconnected(&server, error, &self.config, sent_time);
+                        dashboard
+                            .broadcast(
+                                &server,
+                                &self.config,
+                                sent_time,
+                                Broadcast::Disconnected { error },
+                            )
+                            .map(Message::Dashboard)
                     }
-
-                    Task::none()
                 }
                 stream::Update::Connected {
                     server,
@@ -427,14 +452,16 @@ impl Halloy {
                     if is_initial {
                         notification::connected(&self.config.notifications, &server);
 
-                        dashboard.broadcast_connected(&server, &self.config, sent_time);
+                        dashboard
+                            .broadcast(&server, &self.config, sent_time, Broadcast::Connected)
+                            .map(Message::Dashboard)
                     } else {
                         notification::reconnected(&self.config.notifications, &server);
 
-                        dashboard.broadcast_reconnected(&server, &self.config, sent_time);
+                        dashboard
+                            .broadcast(&server, &self.config, sent_time, Broadcast::Reconnected)
+                            .map(Message::Dashboard)
                     }
-
-                    Task::none()
                 }
                 stream::Update::ConnectionFailed {
                     server,
@@ -445,9 +472,14 @@ impl Halloy {
                         return Task::none();
                     };
 
-                    dashboard.broadcast_connection_failed(&server, error, &self.config, sent_time);
-
-                    Task::none()
+                    dashboard
+                        .broadcast(
+                            &server,
+                            &self.config,
+                            sent_time,
+                            Broadcast::ConnectionFailed { error },
+                        )
+                        .map(Message::Dashboard)
                 }
                 stream::Update::MessagesReceived(server, messages) => {
                     let Screen::Dashboard(dashboard) = &mut self.screen else {
@@ -480,7 +512,11 @@ impl Halloy {
                                             resolve_user_attributes,
                                             channel_users,
                                         ) {
-                                            dashboard.record_message(&server, message);
+                                            commands.push(
+                                                dashboard
+                                                    .record_message(&server, message)
+                                                    .map(Message::Dashboard),
+                                            );
                                         }
                                     }
                                     data::client::Event::WithTarget(encoded, our_nick, target) => {
@@ -491,9 +527,13 @@ impl Halloy {
                                             resolve_user_attributes,
                                             channel_users,
                                         ) {
-                                            dashboard.record_message(
-                                                &server,
-                                                message.with_target(target),
+                                            commands.push(
+                                                dashboard
+                                                    .record_message(
+                                                        &server,
+                                                        message.with_target(target),
+                                                    )
+                                                    .map(Message::Dashboard),
                                             );
                                         }
                                     }
@@ -503,16 +543,20 @@ impl Halloy {
                                             comment,
                                             channels,
                                             sent_time,
-                                        } => {
-                                            dashboard.broadcast_quit(
-                                                &server,
-                                                user,
-                                                comment,
-                                                channels,
-                                                &self.config,
-                                                sent_time,
-                                            );
-                                        }
+                                        } => commands.push(
+                                            dashboard
+                                                .broadcast(
+                                                    &server,
+                                                    &self.config,
+                                                    sent_time,
+                                                    Broadcast::Quit {
+                                                        user,
+                                                        comment,
+                                                        user_channels: channels,
+                                                    },
+                                                )
+                                                .map(Message::Dashboard),
+                                        ),
                                         data::client::Broadcast::Nickname {
                                             old_user,
                                             new_nick,
@@ -522,14 +566,20 @@ impl Halloy {
                                         } => {
                                             let old_nick = old_user.nickname();
 
-                                            dashboard.broadcast_nickname(
-                                                &server,
-                                                old_nick.to_owned(),
-                                                new_nick,
-                                                ourself,
-                                                channels,
-                                                &self.config,
-                                                sent_time,
+                                            commands.push(
+                                                dashboard
+                                                    .broadcast(
+                                                        &server,
+                                                        &self.config,
+                                                        sent_time,
+                                                        Broadcast::Nickname {
+                                                            old_nick: old_nick.to_owned(),
+                                                            new_nick,
+                                                            ourself,
+                                                            user_channels: channels,
+                                                        },
+                                                    )
+                                                    .map(Message::Dashboard),
                                             );
                                         }
                                         data::client::Broadcast::Invite {
@@ -540,13 +590,19 @@ impl Halloy {
                                         } => {
                                             let inviter = inviter.nickname();
 
-                                            dashboard.broadcast_invite(
-                                                &server,
-                                                inviter.to_owned(),
-                                                channel,
-                                                user_channels,
-                                                &self.config,
-                                                sent_time,
+                                            commands.push(
+                                                dashboard
+                                                    .broadcast(
+                                                        &server,
+                                                        &self.config,
+                                                        sent_time,
+                                                        Broadcast::Invite {
+                                                            inviter: inviter.to_owned(),
+                                                            channel,
+                                                            user_channels,
+                                                        },
+                                                    )
+                                                    .map(Message::Dashboard),
                                             );
                                         }
                                         data::client::Broadcast::ChangeHost {
@@ -557,15 +613,21 @@ impl Halloy {
                                             channels,
                                             sent_time,
                                         } => {
-                                            dashboard.broadcast_change_host(
-                                                &server,
-                                                old_user,
-                                                new_username,
-                                                new_hostname,
-                                                ourself,
-                                                channels,
-                                                &self.config,
-                                                sent_time,
+                                            commands.push(
+                                                dashboard
+                                                    .broadcast(
+                                                        &server,
+                                                        &self.config,
+                                                        sent_time,
+                                                        Broadcast::ChangeHost {
+                                                            old_user,
+                                                            new_username,
+                                                            new_hostname,
+                                                            ourself,
+                                                            user_channels: channels,
+                                                        },
+                                                    )
+                                                    .map(Message::Dashboard),
                                             );
                                         }
                                     },
@@ -581,7 +643,26 @@ impl Halloy {
                                             resolve_user_attributes,
                                             channel_users,
                                         ) {
-                                            dashboard.record_message(&server, message);
+                                            commands.push(
+                                                dashboard
+                                                    .record_message(&server, message.clone())
+                                                    .map(Message::Dashboard),
+                                            );
+
+                                            if matches!(
+                                                notification,
+                                                data::client::Notification::Highlight { .. }
+                                            ) {
+                                                commands.extend(
+                                                    message.into_highlight(server.clone()).map(
+                                                        |message| {
+                                                            dashboard
+                                                                .record_highlight(message)
+                                                                .map(Message::Dashboard)
+                                                        },
+                                                    ),
+                                                );
+                                            }
                                         }
 
                                         match notification {
@@ -589,8 +670,8 @@ impl Halloy {
                                                 // only send notification if query has unread
                                                 // or if window is not focused
                                                 if dashboard.history().has_unread(
-                                                    &server,
                                                     &history::Kind::Query(
+                                                        server.clone(),
                                                         user.nickname().to_owned(),
                                                     ),
                                                 ) || !self.main_window.focused
@@ -601,15 +682,18 @@ impl Halloy {
                                                     );
                                                 }
                                             }
-                                            data::client::Notification::Highlight(
+                                            data::client::Notification::Highlight {
+                                                enabled,
                                                 user,
                                                 channel,
-                                            ) => {
-                                                notification::highlight(
-                                                    &self.config.notifications,
-                                                    user.nickname(),
-                                                    channel,
-                                                );
+                                            } => {
+                                                if enabled {
+                                                    notification::highlight(
+                                                        &self.config.notifications,
+                                                        user.nickname(),
+                                                        channel,
+                                                    );
+                                                }
                                             }
                                             data::client::Notification::MonitoredOnline(
                                                 targets,
@@ -644,6 +728,26 @@ impl Halloy {
                                             commands.push(command.map(Message::Dashboard));
                                         }
                                     }
+                                    data::client::Event::UpdateReadMarker(target, read_marker) => {
+                                        commands.push(
+                                            dashboard
+                                                .update_read_marker(
+                                                    history::Kind::from_target(
+                                                        server.clone(),
+                                                        target,
+                                                    ),
+                                                    read_marker,
+                                                )
+                                                .map(Message::Dashboard),
+                                        );
+                                    }
+                                    data::client::Event::JoinedChannel(channel) => {
+                                        commands.push(
+                                            dashboard
+                                                .channel_joined(server.clone(), channel)
+                                                .map(Message::Dashboard),
+                                        );
+                                    }
                                 }
                             }
 
@@ -657,46 +761,56 @@ impl Halloy {
 
                     Task::batch(commands)
                 }
-                stream::Update::Quit(server, reason) => {
-                    let Screen::Dashboard(dashboard) = &mut self.screen else {
-                        return Task::none();
-                    };
+                stream::Update::Quit(server, reason) => match &mut self.screen {
+                    Screen::Dashboard(dashboard) => {
+                        self.servers.remove(&server);
 
-                    self.servers.remove(&server);
+                        if let Some(client) = self.clients.remove(&server) {
+                            let user = client.nickname().to_owned().into();
 
-                    if let Some(client) = self.clients.remove(&server) {
-                        let user = client.nickname().to_owned().into();
+                            let channels = client.channels().to_vec();
 
-                        let channels = client.channels().to_vec();
-
-                        dashboard.broadcast_quit(
-                            &server,
-                            user,
-                            reason,
-                            channels,
-                            &self.config,
-                            Utc::now(),
-                        );
+                            dashboard
+                                .broadcast(
+                                    &server,
+                                    &self.config,
+                                    Utc::now(),
+                                    Broadcast::Quit {
+                                        user,
+                                        comment: reason,
+                                        user_channels: channels,
+                                    },
+                                )
+                                .map(Message::Dashboard)
+                        } else {
+                            Task::none()
+                        }
                     }
+                    Screen::Exit { pending_exit } => {
+                        pending_exit.remove(&server);
 
-                    Task::none()
-                }
+                        if pending_exit.is_empty() {
+                            iced::exit()
+                        } else {
+                            Task::none()
+                        }
+                    }
+                    _ => Task::none(),
+                },
             },
             Message::Event(window, event) => {
-                // Events only enabled for main window
-                if window == self.main_window.id {
-                    if let Screen::Dashboard(dashboard) = &mut self.screen {
-                        return dashboard
-                            .handle_event(
-                                event,
-                                &self.clients,
-                                &self.version,
-                                &self.config,
-                                &mut self.theme,
-                                &self.main_window,
-                            )
-                            .map(Message::Dashboard);
-                    }
+                if let Screen::Dashboard(dashboard) = &mut self.screen {
+                    return dashboard
+                        .handle_event(
+                            window,
+                            event,
+                            &self.clients,
+                            &self.version,
+                            &self.config,
+                            &mut self.theme,
+                            &self.main_window,
+                        )
+                        .map(Message::Dashboard);
                 }
 
                 Task::none()
@@ -764,7 +878,7 @@ impl Halloy {
                         }
                         window::Event::CloseRequested => {
                             if let Screen::Dashboard(dashboard) = &mut self.screen {
-                                return dashboard.exit().then(|_| iced::exit());
+                                return dashboard.exit().map(Message::Dashboard);
                             } else {
                                 return iced::exit();
                             }
@@ -790,6 +904,40 @@ impl Halloy {
 
                 Task::none()
             }
+            Message::AppearanceChange(mode) => {
+                if let data::appearance::Selected::Dynamic { light, dark } =
+                    &self.config.appearance.selected
+                {
+                    self.theme = match mode {
+                        appearance::Mode::Dark => dark.clone().into(),
+                        appearance::Mode::Light => light.clone().into(),
+                    }
+                }
+
+                Task::none()
+            }
+            Message::Logging(mut records) => {
+                let Screen::Dashboard(dashboard) = &mut self.screen else {
+                    self.pending_logs.extend(records);
+
+                    return Task::none();
+                };
+
+                // We've moved from non-dashboard screen to dashboard, prepend records
+                if !self.pending_logs.is_empty() {
+                    records = mem::take(&mut self.pending_logs)
+                        .into_iter()
+                        .chain(records)
+                        .collect();
+                }
+
+                Task::batch(
+                    records
+                        .into_iter()
+                        .map(|record| dashboard.record_log(record)),
+                )
+                .map(Message::Dashboard)
+            }
         }
     }
 
@@ -808,6 +956,7 @@ impl Halloy {
                 Screen::Help(help) => help.view().map(Message::Help),
                 Screen::Welcome(welcome) => welcome.view().map(Message::Welcome),
                 Screen::Migration(migration) => migration.view().map(Message::Migration),
+                Screen::Exit { .. } => column![].into(),
             };
 
             let content = container(screen)
@@ -871,6 +1020,9 @@ impl Halloy {
             url::listen().map(Message::RouteReceived),
             events().map(|(window, event)| Message::Event(window, event)),
             window::events().map(|(window, event)| Message::Window(window, event)),
+            // Enable once dark_light has a proper way to detect appereance changes without spiking CPU.
+            // See: https://github.com/frewsxcv/rust-dark-light/issues/47
+            // appearance::subscription().map(Message::AppearanceChange),
             tick,
             streams,
         ])
