@@ -1,9 +1,9 @@
 use data::config::{self, sidebar, Config};
 use data::dashboard::{BufferAction, BufferFocusedAction};
-use data::{file_transfer, history, Buffer, Version};
+use data::{buffer, file_transfer, history, Version};
 use iced::widget::{
-    button, center, column, container, horizontal_space, pane_grid, row, scrollable, text,
-    vertical_rule, vertical_space, Column, Row, Scrollable,
+    button, column, container, horizontal_rule, horizontal_space, pane_grid, row, scrollable, text,
+    vertical_rule, vertical_space, Column, Row, Scrollable, Space,
 };
 use iced::{padding, Alignment, Length, Task};
 use std::time::Duration;
@@ -11,42 +11,45 @@ use std::time::Duration;
 use tokio::time;
 
 use super::Panes;
-use crate::widget::{context_menu, tooltip, Element};
+use crate::widget::{context_menu, Element, Text};
 use crate::{icon, theme, window};
 
 const CONFIG_RELOAD_DELAY: Duration = Duration::from_secs(1);
 
 #[derive(Debug, Clone)]
 pub enum Message {
-    Open(Buffer),
-    Popout(Buffer),
+    Open(buffer::Upstream),
+    Popout(buffer::Upstream),
     Focus(window::Id, pane_grid::Pane),
-    Replace(window::Id, Buffer, pane_grid::Pane),
+    Replace(window::Id, buffer::Upstream, pane_grid::Pane),
     Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane, window::Id, pane_grid::Pane),
-    Leave(Buffer),
-    ToggleFileTransfers,
+    Leave(buffer::Upstream),
+    ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
     ToggleThemeEditor,
     ReloadingConfigFile,
     ConfigReloaded(Result<Config, config::Error>),
     OpenReleaseWebsite,
+    OpenDocumentation,
     ReloadComplete,
+    Noop,
 }
 
 #[derive(Debug, Clone)]
 pub enum Event {
-    Open(Buffer),
-    Popout(Buffer),
+    Open(buffer::Upstream),
+    Popout(buffer::Upstream),
     Focus(window::Id, pane_grid::Pane),
-    Replace(window::Id, Buffer, pane_grid::Pane),
+    Replace(window::Id, buffer::Upstream, pane_grid::Pane),
     Close(window::Id, pane_grid::Pane),
     Swap(window::Id, pane_grid::Pane, window::Id, pane_grid::Pane),
-    Leave(Buffer),
-    ToggleFileTransfers,
+    Leave(buffer::Upstream),
+    ToggleInternalBuffer(buffer::Internal),
     ToggleCommandBar,
     ToggleThemeEditor,
     OpenReleaseWebsite,
+    OpenDocumentation,
     ConfigReloaded(Result<Config, config::Error>),
 }
 
@@ -88,7 +91,9 @@ impl Sidebar {
                 Some(Event::Swap(from_window, from_pane, to_window, to_pane)),
             ),
             Message::Leave(buffer) => (Task::none(), Some(Event::Leave(buffer))),
-            Message::ToggleFileTransfers => (Task::none(), Some(Event::ToggleFileTransfers)),
+            Message::ToggleInternalBuffer(buffer) => {
+                (Task::none(), Some(Event::ToggleInternalBuffer(buffer)))
+            }
             Message::ToggleCommandBar => (Task::none(), Some(Event::ToggleCommandBar)),
             Message::ToggleThemeEditor => (Task::none(), Some(Event::ToggleThemeEditor)),
             Message::ReloadingConfigFile => {
@@ -106,8 +111,130 @@ impl Sidebar {
                 self.reloading_config = false;
                 (Task::none(), None)
             }
+            Message::Noop => (Task::none(), None),
+            Message::OpenDocumentation => (Task::none(), Some(Event::OpenDocumentation)),
         }
     }
+
+    fn user_menu_button<'a>(
+        &self,
+        keyboard: &'a data::config::Keyboard,
+        file_transfers: &'a file_transfer::Manager,
+        version: &'a Version,
+    ) -> Element<'a, Message> {
+        let base = button(icon::menu())
+            .padding(5)
+            .width(Length::Shrink)
+            .on_press(Message::Noop);
+
+        let menu = Menu::list();
+
+        if menu.is_empty() {
+            base.into()
+        } else {
+            context_menu(
+                context_menu::MouseButton::Left,
+                base,
+                menu,
+                move |menu, length| {
+                    let context_button =
+                        |title: Text<'a>,
+                         keybind: Option<&data::shortcut::KeyBind>,
+                         icon: Text<'a>,
+                         message: Message| {
+                            button(
+                                row![icon.width(Length::Fixed(12.0)), title]
+                                    .push_maybe(keybind.map(|kb| {
+                                        text(format!("({kb})"))
+                                            .shaping(text::Shaping::Advanced)
+                                            .size(theme::TEXT_SIZE - 2.0)
+                                            .style(theme::text::secondary)
+                                    }))
+                                    .spacing(8)
+                                    .align_y(iced::Alignment::Center),
+                            )
+                            .width(length)
+                            .padding(5)
+                            .on_press(message)
+                            .into()
+                        };
+
+                    match menu {
+                        Menu::RefreshConfig => context_button(
+                            text("Reload configuration"),
+                            Some(&keyboard.reload_configuration),
+                            icon::refresh(),
+                            Message::ReloadingConfigFile,
+                        ),
+                        Menu::CommandBar => context_button(
+                            text("Command Bar"),
+                            Some(&keyboard.command_bar),
+                            icon::search(),
+                            Message::ToggleCommandBar,
+                        ),
+                        Menu::FileTransfers => context_button(
+                            text("File Transfers").style(if file_transfers.is_empty() {
+                                theme::text::primary
+                            } else {
+                                theme::text::tertiary
+                            }),
+                            Some(&keyboard.file_transfers),
+                            icon::file_transfer().style(if file_transfers.is_empty() {
+                                theme::text::primary
+                            } else {
+                                theme::text::tertiary
+                            }),
+                            Message::ToggleInternalBuffer(buffer::Internal::FileTransfers),
+                        ),
+                        Menu::Highlights => context_button(
+                            text("Highlights"),
+                            Some(&keyboard.highlight),
+                            icon::highlights(),
+                            Message::ToggleInternalBuffer(buffer::Internal::Highlights),
+                        ),
+                        Menu::Logs => context_button(
+                            text("Logs"),
+                            Some(&keyboard.logs),
+                            icon::logs(),
+                            Message::ToggleInternalBuffer(buffer::Internal::Logs),
+                        ),
+                        Menu::ThemeEditor => context_button(
+                            text("Theme Editor"),
+                            Some(&keyboard.theme_editor),
+                            icon::theme_editor(),
+                            Message::ToggleThemeEditor,
+                        ),
+                        Menu::HorizontalRule => match length {
+                            Length::Fill => container(horizontal_rule(1)).padding([0, 6]).into(),
+                            _ => Space::new(length, 1).into(),
+                        },
+                        Menu::Version => match version.is_old() {
+                            true => context_button(
+                                text("New version available").style(theme::text::tertiary),
+                                None,
+                                icon::megaphone().style(theme::text::tertiary),
+                                Message::OpenReleaseWebsite,
+                            ),
+                            false => container(
+                                text(format!("Halloy ({})", version.current))
+                                    .style(theme::text::secondary),
+                            )
+                            .padding(5)
+                            .into(),
+                        },
+                        Menu::Documentation => context_button(
+                            text("Documentation"),
+                            None,
+                            icon::documentation(),
+                            Message::OpenDocumentation,
+                        ),
+                    }
+                },
+            )
+            .into()
+        }
+    }
+
     pub fn view<'a>(
         &'a self,
         clients: &data::client::Map,
@@ -115,87 +242,82 @@ impl Sidebar {
         panes: &'a Panes,
         focus: Option<(window::Id, pane_grid::Pane)>,
         config: data::config::Sidebar,
-        show_tooltips: bool,
+        keyboard: &'a data::config::Keyboard,
         file_transfers: &'a file_transfer::Manager,
         version: &'a Version,
-        theme_editor_open: bool,
         main_window: window::Id,
     ) -> Option<Element<'a, Message>> {
         if self.hidden {
             return None;
         }
 
-        let menu_buttons = menu_buttons(
-            main_window,
-            panes,
-            config,
-            show_tooltips,
-            file_transfers,
-            version,
-            theme_editor_open,
-            self.reloading_config,
-        );
+        let user_menu_button = config
+            .show_user_menu
+            .then(|| self.user_menu_button(keyboard, file_transfers, version));
 
         let mut buffers = vec![];
 
         for (i, (server, state)) in clients.iter().enumerate() {
             match state {
                 data::client::State::Disconnected => {
-                    buffers.push(buffer_button(
+                    buffers.push(upstream_buffer_button(
                         main_window,
                         panes,
                         focus,
-                        Buffer::Server(server.clone()),
+                        buffer::Upstream::Server(server.clone()),
                         false,
                         config.buffer_action,
                         config.buffer_focused_action,
                         config.position,
                         config.unread_indicator,
-                        history.has_unread(server, &history::Kind::Server),
+                        history.has_unread(&history::Kind::Server(server.clone())),
                     ));
                 }
                 data::client::State::Ready(connection) => {
-                    buffers.push(buffer_button(
+                    buffers.push(upstream_buffer_button(
                         main_window,
                         panes,
                         focus,
-                        Buffer::Server(server.clone()),
+                        buffer::Upstream::Server(server.clone()),
                         true,
                         config.buffer_action,
                         config.buffer_focused_action,
                         config.position,
                         config.unread_indicator,
-                        history.has_unread(server, &history::Kind::Server),
+                        history.has_unread(&history::Kind::Server(server.clone())),
                     ));
 
                     for channel in connection.channels() {
-                        buffers.push(buffer_button(
+                        buffers.push(upstream_buffer_button(
                             main_window,
                             panes,
                             focus,
-                            Buffer::Channel(server.clone(), channel.clone()),
+                            buffer::Upstream::Channel(server.clone(), channel.clone()),
                             true,
                             config.buffer_action,
                             config.buffer_focused_action,
                             config.position,
                             config.unread_indicator,
-                            history.has_unread(server, &history::Kind::Channel(channel.clone())),
+                            history.has_unread(&history::Kind::Channel(
+                                server.clone(),
+                                channel.clone(),
+                            )),
                         ));
                     }
 
                     let queries = history.get_unique_queries(server);
                     for user in queries {
-                        buffers.push(buffer_button(
+                        buffers.push(upstream_buffer_button(
                             main_window,
                             panes,
                             focus,
-                            Buffer::Query(server.clone(), user.clone()),
+                            buffer::Upstream::Query(server.clone(), user.clone()),
                             true,
                             config.buffer_action,
                             config.buffer_focused_action,
                             config.position,
                             config.unread_indicator,
-                            history.has_unread(server, &history::Kind::Query(user.clone())),
+                            history.has_unread(&history::Kind::Query(server.clone(), user.clone())),
                         ));
                     }
 
@@ -227,7 +349,8 @@ impl Sidebar {
                             .scroller_width(0),
                     )),];
 
-                let body = column![container(content).height(Length::Fill), menu_buttons];
+                let body =
+                    column![container(content).height(Length::Fill)].push_maybe(user_menu_button);
                 let padding = match config.position {
                     sidebar::Position::Left => padding::top(8).bottom(6).left(6),
                     sidebar::Position::Right => padding::top(8).bottom(6).right(6),
@@ -251,9 +374,9 @@ impl Sidebar {
                             .scroller_width(0),
                     )),];
 
-                let body: Row<Message, theme::Theme> =
-                    row![container(content).width(Length::Fill), menu_buttons]
-                        .align_y(Alignment::Center);
+                let body: Row<Message, theme::Theme> = row![container(content).width(Length::Fill)]
+                    .push_maybe(user_menu_button)
+                    .align_y(Alignment::Center);
                 let padding = match config.position {
                     sidebar::Position::Top => padding::top(8).left(8).right(8),
                     sidebar::Position::Bottom => padding::bottom(8).left(8).right(8),
@@ -271,6 +394,35 @@ impl Sidebar {
     }
 }
 
+#[derive(Debug, Clone, Copy)]
+enum Menu {
+    RefreshConfig,
+    CommandBar,
+    ThemeEditor,
+    Highlights,
+    Logs,
+    FileTransfers,
+    Version,
+    HorizontalRule,
+    Documentation,
+}
+
+impl Menu {
+    fn list() -> Vec<Self> {
+        vec![
+            Menu::Version,
+            Menu::HorizontalRule,
+            Menu::CommandBar,
+            Menu::FileTransfers,
+            Menu::Highlights,
+            Menu::Logs,
+            Menu::RefreshConfig,
+            Menu::ThemeEditor,
+            Menu::Documentation,
+        ]
+    }
+}
+
 #[derive(Debug, Clone, Copy)]
 enum Entry {
     NewPane,
@@ -312,11 +464,11 @@ impl Entry {
     }
 }
 
-fn buffer_button(
+fn upstream_buffer_button(
     main_window: window::Id,
     panes: &Panes,
     focus: Option<(window::Id, pane_grid::Pane)>,
-    buffer: Buffer,
+    buffer: buffer::Upstream,
     connected: bool,
     buffer_action: BufferAction,
     focused_buffer_action: Option<BufferFocusedAction>,
@@ -327,12 +479,12 @@ fn buffer_button(
     let open = panes
         .iter(main_window)
         .find_map(|(window_id, pane, state)| {
-            (state.buffer.data() == Some(&buffer)).then_some((window_id, pane))
+            (state.buffer.upstream() == Some(&buffer)).then_some((window_id, pane))
         });
     let is_focused = panes
         .iter(main_window)
         .find_map(|(window_id, pane, state)| {
-            (Some((window_id, pane)) == focus && state.buffer.data() == Some(&buffer))
+            (Some((window_id, pane)) == focus && state.buffer.upstream() == Some(&buffer))
                 .then_some((window_id, pane))
         });
 
@@ -364,7 +516,7 @@ fn buffer_button(
     };
 
     let row = match &buffer {
-        Buffer::Server(server) => row![
+        buffer::Upstream::Server(server) => row![
             icon::connected().style(if connected {
                 if show_unread_indicator {
                     theme::text::unread_indicator
@@ -380,7 +532,7 @@ fn buffer_button(
         ]
         .spacing(8)
         .align_y(iced::Alignment::Center),
-        Buffer::Channel(_, channel) => row![]
+        buffer::Upstream::Channel(_, channel) => row![]
             .push(horizontal_space().width(3))
             .push_maybe(
                 show_unread_indicator
@@ -394,7 +546,7 @@ fn buffer_button(
             )
             .push(horizontal_space().width(3))
             .align_y(iced::Alignment::Center),
-        Buffer::Query(_, nick) => row![]
+        buffer::Upstream::Query(_, nick) => row![]
             .push(horizontal_space().width(3))
             .push_maybe(
                 show_unread_indicator
@@ -457,7 +609,7 @@ fn buffer_button(
     if entries.is_empty() || !connected {
         base.into()
     } else {
-        context_menu(base, entries, move |entry, length| {
+        context_menu(Default::default(), base, entries, move |entry, length| {
             let (content, message) = match entry {
                 Entry::NewPane => ("Open in new pane", Message::Open(buffer.clone())),
                 Entry::Popout => ("Open in new window", Message::Popout(buffer.clone())),
@@ -472,9 +624,9 @@ fn buffer_button(
                 ),
                 Entry::Leave => (
                     match &buffer {
-                        Buffer::Server(_) => "Leave server",
-                        Buffer::Channel(_, _) => "Leave channel",
-                        Buffer::Query(_, _) => "Close query",
+                        buffer::Upstream::Server(_) => "Leave server",
+                        buffer::Upstream::Channel(_, _) => "Leave channel",
+                        buffer::Upstream::Query(_, _) => "Close query",
                     },
                     Message::Leave(buffer.clone()),
                 ),
@@ -489,142 +641,3 @@ fn buffer_button(
         .into()
     }
 }
-
-fn menu_buttons<'a>(
-    main_window: window::Id,
-    panes: &Panes,
-    config: data::config::Sidebar,
-    show_tooltips: bool,
-    file_transfers: &'a file_transfer::Manager,
-    version: &'a Version,
-    theme_editor_open: bool,
-    reloading_config: bool,
-) -> Element<'a, Message> {
-    let mut menu_buttons = row![]
-        .spacing(1)
-        .width(Length::Shrink)
-        .align_y(Alignment::End);
-
-    let tooltip_position = match config.position {
-        sidebar::Position::Top => tooltip::Position::Bottom,
-        sidebar::Position::Bottom | sidebar::Position::Left | sidebar::Position::Right => {
-            tooltip::Position::Top
-        }
-    };
-
-    if version.is_old() {
-        let button = button(center(icon::megaphone().style(theme::text::tertiary)))
-            .on_press(Message::OpenReleaseWebsite)
-            .padding(5)
-            .width(22)
-            .height(22)
-            .style(|theme, status| theme::button::primary(theme, status, false));
-
-        let button_with_tooltip = tooltip(
-            button,
-            show_tooltips.then_some("New Halloy version is available!"),
-            tooltip_position,
-        );
-
-        menu_buttons = menu_buttons.push(button_with_tooltip);
-    }
-
-    if config.buttons.reload_config {
-        let icon = if reloading_config {
-            icon::checkmark().style(theme::text::success)
-        } else {
-            icon::refresh().style(theme::text::primary)
-        };
-
-        let button = button(center(icon))
-            .on_press_maybe(if reloading_config {
-                None
-            } else {
-                Some(Message::ReloadingConfigFile)
-            })
-            .padding(5)
-            .width(22)
-            .height(22)
-            .style(|theme, status| theme::button::primary(theme, status, false));
-
-        let button_with_tooltip = tooltip(
-            button,
-            show_tooltips.then_some("Reload config file"),
-            tooltip_position,
-        );
-
-        menu_buttons = menu_buttons.push(button_with_tooltip);
-    }
-
-    if config.buttons.command_bar {
-        let button = button(center(icon::search()))
-            .on_press(Message::ToggleCommandBar)
-            .padding(5)
-            .width(22)
-            .height(22)
-            .style(|theme, status| theme::button::primary(theme, status, false));
-
-        let button_with_tooltip = tooltip(
-            button,
-            show_tooltips.then_some("Command Bar"),
-            tooltip_position,
-        );
-
-        menu_buttons = menu_buttons.push(button_with_tooltip);
-    }
-
-    if config.buttons.file_transfer {
-        let file_transfers_open = panes
-            .iter(main_window)
-            .any(|(_, _, pane)| matches!(pane.buffer, crate::buffer::Buffer::FileTransfers(_)));
-
-        let button = button(center(icon::file_transfer().style(
-            if file_transfers.is_empty() {
-                theme::text::primary
-            } else {
-                theme::text::action
-            },
-        )))
-        .on_press(Message::ToggleFileTransfers)
-        .padding(5)
-        .width(22)
-        .height(22)
-        .style(move |theme, status| theme::button::primary(theme, status, file_transfers_open));
-
-        let button_with_tooltip = tooltip(
-            button,
-            show_tooltips.then_some("File Transfers"),
-            tooltip_position,
-        );
-
-        menu_buttons = menu_buttons.push(button_with_tooltip);
-    }
-
-    if config.buttons.theme_editor {
-        let button = button(center(icon::theme_editor().style(theme::text::primary)))
-            .on_press(Message::ToggleThemeEditor)
-            .padding(5)
-            .width(22)
-            .height(22)
-            .style(move |theme, status| theme::button::primary(theme, status, theme_editor_open));
-
-        let button_with_tooltip = tooltip(
-            button,
-            show_tooltips.then_some("Theme Editor"),
-            tooltip_position,
-        );
-
-        menu_buttons = menu_buttons.push(button_with_tooltip);
-    }
-
-    let width = if config.position.is_horizontal() {
-        Length::Shrink
-    } else {
-        Length::Fill
-    };
-
-    container(menu_buttons)
-        .width(width)
-        .align_x(Alignment::Center)
-        .into()
-}
